/**
 * Created by janis dähne.
 */
import {DoExerciseFromBackend} from "../types/doExercise";
import * as genericLayer from './genericLayer'
import {AssessmentForUserWithTestResults, StoredTestResultTupleFromBackend} from "../types/testResults";
import {SolutionDoExerciseFullBase, SolutionTupleFromBackend} from "../types/exerciseSolution";
import {
  CustomTestForBackend,
  CustomTestFromBackend,
  RunnableCustomTestFrontendExclusive,
  UpdateCustomTestIndicesTuple
} from "../types/customTests";
import {CancelToken} from 'axios'
import {FilePreviewFromBackend} from '../types/exerciseEditor'


const callingName = 'doExerciseLayer'
const controllerPrefix = 'exercise/do'

/**
 * gets the exercise for the release (this also ensures that the user has a participation & solution & 1 solution versions)
 * @param releaseCode the release code (identifies the release)
 */
export async function getDoExercise(releaseCode: string): Promise<DoExerciseFromBackend> {
  return genericLayer.get<DoExerciseFromBackend>(callingName,
    `${controllerPrefix}/${releaseCode}`)
}

/**
 * returns a solution version for the exercise (identified by the release code & plang)
 * @param releaseCode the exercise release code
 * @param plangId the plang
 * @returns {Promise<SolutionDoExerciseFullBase>}
 */
export async function getDoExerciseSolution(releaseCode: string, plangId: number): Promise<SolutionTupleFromBackend> {

  return genericLayer.get<SolutionTupleFromBackend>(callingName,
                                                    `${controllerPrefix}/solution/${releaseCode}/${plangId}`)
}

/**
 * sets should not count to true (exercise release participation --> for all plangs/solutions for this release)
 * @param {string} releaseCode
 * @returns {Promise<void>}
 */
export async function setShouldNotCount(releaseCode: string): Promise<void> {

  return genericLayer.update<{}, void>(callingName,
    `${controllerPrefix}/solution/shouldnotcount/${releaseCode}`, {})
}


/**
 * returns the test results for the normal tests & the custom tests (test results are specific for plang)
 * submit tests & results are excluded
 * @param releaseCode the exercise release code
 * @param plangId the plang
 * @returns {Promise<StoredTestResultTupleFromBackend>}
 */
export async function getAllTestResults(releaseCode: string, plangId: number): Promise<StoredTestResultTupleFromBackend> {
  return genericLayer.get<StoredTestResultTupleFromBackend>(callingName,
    `${controllerPrefix}/testresults/${releaseCode}/${plangId}`)
}


/**
 * tries to return the submit test results and manual assessment (only if the release ended else error)
 * @param releaseCode the release code
 * @param pLangId the plang id
 * @returns {Promise<AssessmentForUserWithTestResults>}
 */
export async function getSubmitTestResults(releaseCode: string, pLangId: number): Promise<AssessmentForUserWithTestResults> {
  return genericLayer.get<AssessmentForUserWithTestResults>(callingName,
    `${controllerPrefix}/testresults/submit/${releaseCode}/${pLangId}`)
}


/**
 * saves the given solution version
 *
 * when this method throws (reject) then the error contains the request ... when the request code is
 * InvalidRequest and the data contains a string, then this string a the invalid file name
 * @param releaseCode the exercise release code
 * @param shouldClearAllTestResults
 * @param solutionVersion the solution version to save
 */
export async function saveSolutionVersion(releaseCode: string, shouldClearAllTestResults: boolean, solutionVersion: SolutionDoExerciseFullBase): Promise<SolutionTupleFromBackend> {

  return genericLayer.update<SolutionDoExerciseFullBase, SolutionTupleFromBackend>(callingName,
                                                                                   `${controllerPrefix}/save/${releaseCode}/${shouldClearAllTestResults}`, solutionVersion)
}

/**
 * sets the lock solutions flag on the participation (to the given value)
 * @param releaseCode
 * @param shouldLockSolutions
 */
export async function setLockSolutions(releaseCode: string, shouldLockSolutions: boolean): Promise<boolean> {

  return genericLayer.update<{}, boolean>(callingName,
                                                                                   `${controllerPrefix}/lock/solutions/${releaseCode}/${shouldLockSolutions}`, {})
}

/**
 * resets the code in the backend and returns the new solution
 * @param releaseCode
 * @param pLangId
 */
export async function resetSolution(releaseCode: string, pLangId: number): Promise<SolutionDoExerciseFullBase> {
  return genericLayer.update<any,SolutionDoExerciseFullBase>(callingName,
                                `${controllerPrefix}/reset/${releaseCode}/${pLangId}`, {})
}


/**
 * creates a new custom test
 * @param releaseCode the exercise release code
 * @param customTest the custom test to create
 */
export async function createCustomTest(releaseCode: string, customTest: CustomTestForBackend): Promise<CustomTestFromBackend> {

  return genericLayer.create<CustomTestForBackend, CustomTestFromBackend>(callingName,
    `${controllerPrefix}/create/customtest/${releaseCode}`, customTest)
}

/**
 * updates the given custom test
 * @param releaseCode the exercise release code
 * @param customTest the custom test to create
 */
export async function updateCustomTest(releaseCode: string, customTest: CustomTestForBackend): Promise<CustomTestFromBackend> {
  return genericLayer.update<CustomTestForBackend, CustomTestFromBackend>(callingName,
    `${controllerPrefix}/update/customtest/${releaseCode}`, customTest)
}

/**
 * deletes the given custom tests
 * @param releaseCode the exercise release code
 * @param customTest the test to delete
 * @returns {Promise<void>}
 */
export async function deleteCustomTest(releaseCode: string, customTest: CustomTestForBackend | CustomTestFromBackend): Promise<void> {
  return genericLayer.del<void>(callingName,
    `${controllerPrefix}/delete/customtest/${releaseCode}/${customTest.id}`)
}


/**
 * used to update the custom test display indices
 * @param releaseCodes the exercise release code
 * @param customTests the custom tests
 * @returns {Promise<void>}
 */
export async function updateCustomTestDisplayIndices(releaseCodes: string, customTests: ReadonlyArray<RunnableCustomTestFrontendExclusive>): Promise<void> {

  const tuples: ReadonlyArray<UpdateCustomTestIndicesTuple> = customTests.map(p => {
    return {
      id: p.id,
      newDisplayIndex: p.displayIndex
    }
  })

  return genericLayer.update<{ readonly tuples: ReadonlyArray<UpdateCustomTestIndicesTuple> }, void>(callingName,
    controllerPrefix + '/update/indices/customtest/' + releaseCodes, {tuples: tuples})
}


//--- custom test asset files

export async function uploadCustomTestAssetFile(releaseCodes: string, customTestId: number, file: File,
                                                  cancelToken: CancelToken,
                                                  onProgress: (percentage: number) => void,
): Promise<FilePreviewFromBackend> {
  return genericLayer.uploadFileAsync<FilePreviewFromBackend>(callingName,
                                                              `${controllerPrefix}/upload/create/custom/tests/${releaseCodes}/${customTestId}`,
                                                              file, cancelToken, onProgress
  )
}

export async function getPossibleAllUserFiles(): Promise<ReadonlyArray<FilePreviewFromBackend>> {
  return genericLayer.get<ReadonlyArray<FilePreviewFromBackend>>(callingName,
                                                                 `${controllerPrefix}/upload/get/custom`
  )
}

export async function attachExitingFileToCustomTest(releaseCodes: string, customTestId: number,
                                                      fileReferenceId: number
): Promise<FilePreviewFromBackend> {
  return genericLayer.update<{}, FilePreviewFromBackend>(callingName,
                                                         `${controllerPrefix}/upload/attach/custom/tests/${releaseCodes}/${customTestId}/${fileReferenceId}`,
                                                         {}
  )
}

export async function detachFileFromCustomTest(releaseCode: string, customTestId: number, fileReferenceId: number
): Promise<void> {
  return genericLayer.del<void>(callingName,
                                `${controllerPrefix}/upload/detach/custom/tests/${releaseCode}/${customTestId}/${fileReferenceId}`
  )
}
