/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {AddableUserFrontendExclusive, UserPreviewFromBackend} from "../types/users";
import logger from '../helpers/logger'
import {
  AddUsersToGroupTuplesForBackend,
  ChangeGroupRoleTupleForBackend,
  GroupUserFromBackend,
  RemoveUsersFromGroupTuple,
  SelectableGroupUserFrontend, UserGroupForBackend,
  UserGroupBase, UserGroupWithExerciseCountFromBackend
} from "../types/group";
import {GroupRoleWithPermissionFullBase} from "../types/groupRoles";
import {PaginatedData, PaginationPostData} from '../types/pagination'

const callingName = 'groupsLayer'
const controllerPrefix = 'groups'

/**
 * returns the user groups where the current user is a member
 */
export function getGroupsFromMe(paginationPostData: PaginationPostData): Promise<PaginatedData<UserGroupWithExerciseCountFromBackend>> {
  return genericLayer.update<PaginationPostData, PaginatedData<UserGroupWithExerciseCountFromBackend>>(
    callingName, `${controllerPrefix}/fromuser`, paginationPostData
  )
}


/**
 * returns a single group where the user is a member
 * @param groupId
 */
export function getSingleGroupFromMe(groupId: number): Promise<UserGroupWithExerciseCountFromBackend> {
  return genericLayer.get<UserGroupWithExerciseCountFromBackend>(
    callingName, `${controllerPrefix}/fromuser/${groupId}`
  )
}

/**
 * returns all members of the given group
 * @param groupId the group id
 * @param paginationPostData
 */
export function getMembers(groupId: number,
                           paginationPostData: PaginationPostData
): Promise<PaginatedData<GroupUserFromBackend>> {
  return genericLayer.update<PaginationPostData, PaginatedData<GroupUserFromBackend>>(
    callingName, `${controllerPrefix}/${groupId}/members`, paginationPostData
  )
}

/**
 * creates a new user group
 * @param userGroup the user group to create
 * @returns {Promise<UserGroupForBackend>}
 */
export function create(userGroup: UserGroupForBackend): Promise<UserGroupForBackend> {
  return genericLayer.create(callingName,
                             `${controllerPrefix}/create`, userGroup
  )
}


/**
 * updates the given user group
 * @param userGroup the user group
 * @returns {Promise<UserGroupForBackend>}
 */
export function update(userGroup: UserGroupForBackend): Promise<UserGroupForBackend> {

  return genericLayer.update(callingName,
                             `${controllerPrefix}/update`, userGroup
  )
}


/**
 * deletes the given user group
 * @returns {Promise<void>}
 * @param userGroupId
 */
export function deleteUserGroup(userGroupId: number): Promise<void> {

  return genericLayer.del(callingName,
                          `${controllerPrefix}/delete/${userGroupId}`
  )
}


/**
 * returns the group role for the current user for the given group
 * @param {number} userGroupId
 * @returns {GroupRoleWithPermissionFullBase} the permissions or null (if user is not part of the group)
 */
export function getGroupPermissionFromUserForGroup(userGroupId: number): Promise<GroupRoleWithPermissionFullBase | null> {
  return genericLayer.get<GroupRoleWithPermissionFullBase | null>(callingName,
                                                                  `${controllerPrefix}/role/for/${userGroupId}`
  )
}

/**
 * transforms the given member to member with role
 * @param members the members
 * @param groupRoles all possible group roles
 * @param currentUserId the current user id to set 'isCurrentUser'
 */
export function assignGroupRolesToMembers(members: ReadonlyArray<GroupUserFromBackend>,
                                          groupRoles: ReadonlyArray<GroupRoleWithPermissionFullBase>
): ReadonlyArray<SelectableGroupUserFrontend> {

  const finishedMembers: ReadonlyArray<SelectableGroupUserFrontend> = members.map((member) => {

    const groupRole = groupRoles.find(p => p.id === member.groupRoleId)

    const newMember = {
      ...member,
      isCurrentUser: member.isCurrentUser,
      uiIsSelected: false,
      isDisplayed: true,
      groupRole: (groupRole
                  ? groupRole
                  : null), //find will return undefined we want null
      uiIsChangeGroupRolePopupDisplayed: false
    } as SelectableGroupUserFrontend

    if (member.groupRoleId > 0 && !groupRole) {
      //user has a system role but could not be found...
      logger.error(
        `could not find groupRole for user with id ${member.id}` +
        `and systemrole ${member.groupRoleId}`
      )
    }

    return newMember

  })

  return finishedMembers

}

export function toAddableUser(users: ReadonlyArray<UserPreviewFromBackend>,
                              members: ReadonlyArray<SelectableGroupUserFrontend>
                              ): ReadonlyArray<AddableUserFrontendExclusive> {

  const usersForFrontend: ReadonlyArray<AddableUserFrontendExclusive> = users.map((user) => {

    return {
      ...user,
      uiIsSelected: false,
      uiIsDisplayed: true,
      isAlreadyMember: members.some(p => p.id === user.id) //if we find the user in the members list
    } as AddableUserFrontendExclusive
  })

  return usersForFrontend

}

/**
 * adds the given members to the given group
 * @param addUsersTuple the tuple
 * @returns {Promise<void>}
 */
export function addMembers(addUsersTuple: AddUsersToGroupTuplesForBackend): Promise<void> {
  return genericLayer.update(callingName,
                             `${controllerPrefix}/addmembers`, addUsersTuple
  )
}

/**
 * changes the group role of the given member (is also used to drop the group role for the member)
 * @param tuple the tuple (with member and new role)
 * @returns {Promise<void>}
 */
export function changeMemberRole(tuple: ChangeGroupRoleTupleForBackend): Promise<void> {
  return genericLayer.update(callingName,
                             `${controllerPrefix}/changememberrole`, tuple
  )
}


/**
 * checks if the member can leave the group (can only leave if at least on creator is a member)
 * @param tuple
 */
export function canLeaveGroup(tuple: RemoveUsersFromGroupTuple): Promise<boolean> {
  return genericLayer.update<RemoveUsersFromGroupTuple, boolean>(callingName,
                                                                 `${controllerPrefix}/canleave`, tuple
  )
}

/**
 * removes the given member from the group
 * @param tuple the tuple (all necessary data)
 * @returns {Promise<void>}
 */
export function removeMember(tuple: RemoveUsersFromGroupTuple): Promise<void> {
  return genericLayer.update(callingName,
                             `${controllerPrefix}/removemember`, tuple
  )
}
