import * as React from "react";
import {ButtonProps, Form} from "semantic-ui-react";
import {RootState} from "../../../state/reducers";
import {bindActionCreators, Dispatch} from "redux";
import {connect} from "react-redux";
import {getI18n} from "../../../../i18n/i18nRoot";
import {returntypeof} from 'react-redux-typescript'
import {parseId} from '../../../helpers/convertersAndTransformers'
import {errorDialog} from '../../../helpers/dialogHelper'
import MaterialInput from '../../material/materialInput'
import {HelpPopup} from '../helpPopup'
import {testServerPublicSettingsNotAvailableValue, TestTypeLimits} from '../../../constants'


export interface MyProps {

  /**
   * true if we should use the custom project test limits
   */
  readonly testTypeLimits: TestTypeLimits

  readonly timeoutInMs: number
  readonly compileTimeoutInMs: number
  readonly memoryLimitInKb: number
  readonly maxDiskSpaceInKb: number
  readonly compilerOptions: string

  /**
   * can be null for readonly
   */
  readonly set_timeoutInMs: ((val: number) => void) | null

  /**
   * can be null for readonly
   */
  readonly set_compileTimeoutInMs: ((val: number) => void) | null

  /**
   * can be null for readonly
   */
  readonly set_memoryLimitInKb: ((val: number) => void) | null
  /**
   * can be null for readonly
   */
  readonly set_maxDiskSpaceInKb: ((val: number) => void) | null

  /**
   * can be null for readonly
   */
  readonly set_compilerOptions: ((val: string) => void) | null
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {

    ...props,
    testServerPublicSettings: rootState.globalSystemSettingsState.testServerPublicSettings,
    systemSettings: rootState.globalSystemSettingsState.systemSettings,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({}, dispatch)

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class TestSettingsView extends React.Component<Props & ButtonProps, any> {
  render(): JSX.Element {

    const isTimeoutOverwrittenByTestServerSetting = this.props.timeoutInMs > this.props.testServerPublicSettings.hardTimeoutInMs
    const isCompileTimeoutOverwrittenByTestServerSetting = this.props.compileTimeoutInMs > this.props.testServerPublicSettings.hardCompileTimeoutInMs
    const isMemoryLimitInKbOverwritten = this.props.memoryLimitInKb > this.props.testServerPublicSettings.hardMemoryLimitInKb
    const isMaxDiskSpaceInKbOverwritten = this.props.maxDiskSpaceInKb > this.props.testServerPublicSettings.hardDiskSpaceLimitInKb

    //these are the initial values
    const areTestServerSettingsAvailable = this.props.testServerPublicSettings.hardTimeoutInMs !== testServerPublicSettingsNotAvailableValue
      || this.props.testServerPublicSettings.hardCompileTimeoutInMs !== testServerPublicSettingsNotAvailableValue
      || this.props.testServerPublicSettings.hardMemoryLimitInKb !== testServerPublicSettingsNotAvailableValue
      || this.props.testServerPublicSettings.hardDiskSpaceLimitInKb !== testServerPublicSettingsNotAvailableValue

    return (
      <div>
        <Form.Group widths='equal' className="scrollable" style={{paddingBottom: '0.5em'}}>

          <Form.Field required>

            <div className="flexed-no-wrap">
              <label className="not-text-wrapping">
                {
                  getI18n(this.props.langId, 'Timeout in ms')
                }
              </label>

              {
                (this.props.testTypeLimits === TestTypeLimits.IsNormalTest || this.props.testTypeLimits === TestTypeLimits.IsCustomTest) &&
                this.props.timeoutInMs > this.props.systemSettings.testServerTimeoutInMs &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting test server timeout in ms (testServerTimeoutInMs) which is the request timeout backend to test-server (one request can contain many tests)"
                           )} (${this.props.systemSettings.testServerTimeoutInMs} ${getI18n(this.props.langId,
                                                                                            "ms"
                           )})`} wide/>
              }

              {
                this.props.testTypeLimits === TestTypeLimits.IsSubmitTest && this.props.timeoutInMs > this.props.systemSettings.submitTestServerTimeoutInMs &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting submit test server timeout in ms (submitTestServerTimeoutInMs) which is the request timeout backend to submit test-server (one request can contain many tests)"
                           )} (${this.props.systemSettings.submitTestServerTimeoutInMs} ${getI18n(this.props.langId,
                                                                                                  "ms"
                           )})`}
                           wide
                />
              }

              {
                this.props.testTypeLimits === TestTypeLimits.IsJustRunProgram && this.props.timeoutInMs > this.props.systemSettings.justRunProgramTimeoutInMs &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting custom project test timeout in ms (justRunProgramTimeoutInMs) which is the max time the user program can run"
                           )} (${this.props.systemSettings.justRunProgramTimeoutInMs} ${getI18n(this.props.langId,
                                                                                                  "ms"
                           )})`}
                           wide
                />
              }

              {
                this.props.testTypeLimits === TestTypeLimits.IsCustomProjectTest && this.props.timeoutInMs > this.props.systemSettings.customProjectTestTimeoutInMs &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting just run program timeout in ms (justRunProgramTimeoutInMs) which is the max time the user program can run"
                           )} (${this.props.systemSettings.customProjectTestTimeoutInMs} ${getI18n(this.props.langId,
                                                                                                   "ms"
                           )})`}
                           wide
                />
              }

              {
                areTestServerSettingsAvailable && isTimeoutOverwrittenByTestServerSetting &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the test-server setting timeout in ms (hardTimeoutInMs) which is the max time the user program can run"
                           )} (${this.props.testServerPublicSettings.hardTimeoutInMs} ${getI18n(this.props.langId,
                                                                                                "ms"
                           )})`}
                           wide
                />
              }

              {
                !areTestServerSettingsAvailable &&
                <HelpPopup className="mar-left-half error-colored" icon="exclamation triangle"
                           defaultText={getI18n(this.props.langId, "The test-server settings are not available. Thus this setting could be overwritten by some test-server setting")}
                           wide
                />
              }

            </div>


            <MaterialInput
              type="number"
              disabled={!this.props.set_timeoutInMs}
              value={this.props.timeoutInMs}
              onChange={(e) => {

                if (!this.props.set_timeoutInMs) return

                //checks for > 0
                const limit = parseId(e.currentTarget.value)

                if (limit === null) {
                  errorDialog(getI18n(this.props.langId, 'Error'),
                              getI18n(this.props.langId, 'Could not convert string to int'),
                              this.props.langId
                  )
                  return
                }

                this.props.set_timeoutInMs(limit)
              }}
            />
          </Form.Field>

          <Form.Field required>

            <div className="flexed-no-wrap">
              <label  className="not-text-wrapping">
                {
                  getI18n(this.props.langId, 'Compile timeout in ms')
                }
              </label>

              {
                (this.props.testTypeLimits === TestTypeLimits.IsNormalTest || this.props.testTypeLimits === TestTypeLimits.IsCustomTest) &&
                this.props.compileTimeoutInMs > this.props.systemSettings.testServerTimeoutInMs &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting test server timeout in ms (testServerTimeoutInMs) which is the request timeout backend to test-server (one request can contain many tests)"
                           )} (${this.props.systemSettings.testServerTimeoutInMs} ${getI18n(this.props.langId,
                                                                                            "ms"
                           )})`} wide/>
              }

              {
                this.props.testTypeLimits === TestTypeLimits.IsSubmitTest && this.props.compileTimeoutInMs > this.props.systemSettings.submitTestServerTimeoutInMs &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting submit test server timeout in ms (submitTestServerTimeoutInMs) which is the request timeout backend to submit test-server (one request can contain many tests)"
                           )} (${this.props.systemSettings.submitTestServerTimeoutInMs} ${getI18n(this.props.langId,
                                                                                                  "ms"
                           )})`}
                           wide
                />
              }

              {
                this.props.testTypeLimits === TestTypeLimits.IsJustRunProgram && this.props.compileTimeoutInMs > this.props.systemSettings.justRunProgramCompileTimeoutInMs &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting just run program compile timeout in ms (justRunProgramCompileTimeoutInMs) which is the max time the user program can compile"
                           )} (${this.props.systemSettings.justRunProgramCompileTimeoutInMs} ${getI18n(this.props.langId,
                                                                                                "ms"
                           )})`}
                           wide
                />
              }

              {
                this.props.testTypeLimits === TestTypeLimits.IsCustomProjectTest && this.props.compileTimeoutInMs > this.props.systemSettings.customProjectTestCompileTimeoutInMs &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting custom project test compile timeout in ms (customProjectTestCompileTimeoutInMs) which is the max time the user program can compile"
                           )} (${this.props.systemSettings.customProjectTestCompileTimeoutInMs} ${getI18n(this.props.langId,
                                                                                                   "ms"
                           )})`}
                           wide
                />
              }

              {
                areTestServerSettingsAvailable && isCompileTimeoutOverwrittenByTestServerSetting &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the test-server setting compile timeout in ms (hardCompileTimeoutInMs) which is the max time the user program can compile"
                           )} (${this.props.testServerPublicSettings.hardCompileTimeoutInMs} ${getI18n(this.props.langId,
                                                                                                "ms"
                           )})`}
                           wide
                />
              }

              {
                !areTestServerSettingsAvailable &&
                <HelpPopup className="mar-left-half error-colored" icon="exclamation triangle"
                           defaultText={getI18n(this.props.langId, "The test-server settings are not available. Thus this setting could be overwritten by some test-server setting")}
                           wide
                />
              }

            </div>


            <MaterialInput
              type="number"
              disabled={!this.props.set_compileTimeoutInMs}
              value={this.props.compileTimeoutInMs}
              onChange={(e) => {

                if (!this.props.set_compileTimeoutInMs) return

                //checks for > 0
                const limit = parseId(e.currentTarget.value)

                if (limit === null) {
                  errorDialog(getI18n(this.props.langId, 'Error'),
                              getI18n(this.props.langId, 'Could not convert string to int'),
                              this.props.langId
                  )
                  return
                }

                this.props.set_compileTimeoutInMs(limit)
              }}
            />
          </Form.Field>

          <Form.Field required>
            <div className="flexed-no-wrap">
              <label  className="not-text-wrapping">
                {
                  getI18n(this.props.langId, 'Max. RAM in kb')
                }
              </label>

              {
                this.props.testTypeLimits === TestTypeLimits.IsJustRunProgram && this.props.memoryLimitInKb > this.props.systemSettings.justRunProgramMemoryLimitInKb &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting just run program max RAM in kb (justRunProgramMemoryLimitInKb) which is the max memory the user program can use"
                           )} (${this.props.systemSettings.justRunProgramMemoryLimitInKb} ${getI18n(this.props.langId, "kb")})`}
                           wide
                />
              }

              {
                this.props.testTypeLimits === TestTypeLimits.IsCustomProjectTest && this.props.memoryLimitInKb > this.props.systemSettings.customProjectTestMemoryLimitInKb &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting custom project test max RAM in kb (customProjectTestMemoryLimitInKb) which is the max memory the user program can use"
                           )} (${this.props.systemSettings.customProjectTestMemoryLimitInKb} ${getI18n(this.props.langId, "kb")})`}
                           wide
                />
              }

              {
                areTestServerSettingsAvailable && isMemoryLimitInKbOverwritten &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the test-server setting max RAM in kb (hardMemoryLimitInKb) which is the max memory the user program can use"
                           )} (${this.props.testServerPublicSettings.hardMemoryLimitInKb} ${getI18n(this.props.langId, "kb")})`}
                           wide
                />
              }

              {
                !areTestServerSettingsAvailable &&
                <HelpPopup className="mar-left-half error-colored" icon="exclamation triangle"
                           defaultText={getI18n(this.props.langId, "The test-server settings are not available. Thus this setting could be overwritten by some test-server setting")}
                           wide
                />
              }
            </div>
            <MaterialInput
              type="number"
              disabled={!this.props.set_memoryLimitInKb}
              value={this.props.memoryLimitInKb}
              onChange={(e) => {

                if (!this.props.set_memoryLimitInKb) return
                //checks for > 0
                const limit = parseId(e.currentTarget.value)

                if (limit === null) {
                  errorDialog(getI18n(this.props.langId, 'Error'),
                              getI18n(this.props.langId, 'Could not convert string to int'),
                              this.props.langId
                  )
                  return
                }

                this.props.set_memoryLimitInKb(limit)
              }}
            />
          </Form.Field>

          <Form.Field required>
            <div className="flexed-no-wrap">
              <label  className="not-text-wrapping">
                {
                  getI18n(this.props.langId, 'Max disk space in kb')
                }
              </label>

              {
                this.props.testTypeLimits === TestTypeLimits.IsJustRunProgram && this.props.maxDiskSpaceInKb > this.props.systemSettings.justRunProgramMaxDiskSpaceInKb &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting just run program max disk space in kb (justRunProgramMaxDiskSpaceInKb) which is the max disk space the user program can use"
                           )} (${this.props.systemSettings.justRunProgramMaxDiskSpaceInKb} ${getI18n(this.props.langId, "kb")})`}
                           wide
                />
              }

              {
                this.props.testTypeLimits === TestTypeLimits.IsCustomProjectTest && this.props.maxDiskSpaceInKb > this.props.systemSettings.customProjectTestMaxDiskSpaceInKb &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting just run program max disk space in kb (justRunProgramMaxDiskSpaceInKb) which is the max disk space the user program can use"
                           )} (${this.props.systemSettings.customProjectTestMaxDiskSpaceInKb} ${getI18n(this.props.langId, "kb")})`}
                           wide
                />
              }

              {
                areTestServerSettingsAvailable && isMaxDiskSpaceInKbOverwritten &&
                <HelpPopup className="mar-left-half warning-colored" icon="exclamation triangle"
                           defaultText={`${getI18n(this.props.langId,
                                                   "This value is overwritten by the system setting custom project test max disk space in kb (customProjectTestMaxDiskSpaceInKb) which is the max disk space the user program can use"
                           )} (${this.props.testServerPublicSettings.hardDiskSpaceLimitInKb} ${getI18n(this.props.langId, "kb")})`}
                           wide
                />
              }

              {
                !areTestServerSettingsAvailable &&
                <HelpPopup className="mar-left-half error-colored" icon="exclamation triangle"
                           defaultText={getI18n(this.props.langId, "The test-server settings are not available. Thus this setting could be overwritten by some test-server setting")}
                           wide
                />
              }
            </div>
            <MaterialInput
              type="number"
              disabled={!this.props.set_maxDiskSpaceInKb}
              value={this.props.maxDiskSpaceInKb}
              onChange={(e) => {

                if (!this.props.set_maxDiskSpaceInKb) return

                //checks for > 0
                const limit = parseId(e.currentTarget.value)

                if (limit === null) {
                  errorDialog(getI18n(this.props.langId, 'Error'),
                              getI18n(this.props.langId, 'Could not convert string to int'),
                              this.props.langId
                  )
                  return
                }

                this.props.set_maxDiskSpaceInKb(limit)
              }}
            />
          </Form.Field>

          <Form.Field>

            <div className="flexed-no-wrap">
              <label className="not-text-wrapping">
                {
                  getI18n(this.props.langId, 'Compiler options')
                }
              </label>

              <HelpPopup className="mar-left-half" defaultText={getI18n(this.props.langId, "Options passed to the compiler, e.g. for java -Xlint. Multiple arguments must be separated by a whitespace.")} />
            </div>

            <MaterialInput
              disabled={!this.props.set_compilerOptions}
              value={this.props.compilerOptions}
              onChange={(e) => {

                if (!this.props.set_compilerOptions) return

                this.props.set_compilerOptions(e.currentTarget.value)
              }}
            />
          </Form.Field>
        </Form.Group>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TestSettingsView)
