/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers";
import {Input, Icon, InputProps, SemanticICONS} from "semantic-ui-react";
import {ValidationFunc} from "../../validation/validations";
import {CSSProperties, KeyboardEventHandler} from "react";

//const css = require('./styles.styl');

export interface MyProps {
  readonly placeholder?: string
  readonly icon?: SemanticICONS | string
  readonly iconClickHandler?: () => void
  readonly value: string | number | null | undefined
  readonly id?: string
  /**
   * usage: <MaterialInput disabled>
   * https://stackoverflow.com/questions/37828543/how-to-pass-props-without-value-to-component
   */
  readonly disabled?: boolean

  /**
   * true: disallow editing, allow selection (disabled don't allow selection)
   * false: not
   */
  readonly readonly?: boolean

  readonly style?: CSSProperties

  /**
   * ony key press event
   *
   * usage
   * :
   * ... onKeyPress={(e: OnKeyPressEvent) => {...}}
   */
  readonly onKeyPress?: KeyboardEventHandler<HTMLInputElement>

  /**
   * only visually hidden (display: hidden)
   */
  readonly isHidden?: boolean

  /**
   * the validations for the input
   *
   *  usage:
   *
   * <MaterialInput
   *    validations={validationRules.displayName}
   *    validationMessageKey={validationMessageKeys.displayName}
   *  />
   *
   *  validationRules must be specified in the reducer!!
   */
  readonly validations?: ReadonlyArray<ValidationFunc<any>>

  /**
   * the key of the message when the validation fails
   *
   *
   * usage:
   *
   * <MaterialInput
   *    validations={validationRules.displayName}
   *    validationMessageKey={validationMessageKeys.displayName}
   *  />
   *
   * validationMessageKeys must be specified in the reducer!!
   */
  readonly validationMessageKey?: string

  readonly postfixText?: string | null
}

const mapStateToProps = (rootState: RootState, props: MyProps & InputProps) => {
  return {
    ...props
    //inputProps: {...props},
    // placeholder: props.placeholder,
    // icon: props.icon,
    // iconClickHandler: props.iconClickHandler,
    // value: props.value,
    // disabled: props.disabled,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


/**
 *
 * a material input component
 * not all original (semantic ui input) props are compatible e.g. label needs to be provided
 *
 * usage:
 * <div>
 *   <label>xxx</label>
 *   <MaterialInput />
 * </div>
 *
 */
class MaterialInput extends React.Component<Props & InputProps, any> {

  render(): JSX.Element {

    // noinspection TsLint
    let outerStyle: CSSProperties = {}

    if (this.props.style) {
      outerStyle = {
        ...this.props.style
      }
    }

    if (this.props.isHidden === true) {
      outerStyle = {
        ...outerStyle,
        visibility: 'hidden'
      }
    }

    return (
      <div className={this.props.className} style={outerStyle}>
        {
          //note that we pass all props here with
          //https://zhenyong.github.io/react/docs/transferring-props.html or
          //https://zhenyong.github.io/react/docs/jsx-spread.html
          //so we need to reset the className here else it would override the className set in this code
        }
        <Input
          iconPosition={(this.props.icon
                         ? 'left'
                         : null)}
          placeholder={(this.props.placeholder
                        ? this.props.placeholder
                        : null)}
          {...this.props.inputProps}
          icon={undefined}
          className="material-input"
          onFocus={this.props.onFocus}
        >
          {
            this.props.icon &&
            <div className="material-icon-wrapper"
                 onClick={() => this.props.iconClickHandler
                                ? this.props.iconClickHandler()
                                : null}>
              <div>
                <Icon name={this.props.icon}
                      className={this.props.iconClickHandler
                                 ? 'clickable'
                                 : null}/>
              </div>
            </div>
          }

          <input
            id={this.props.id}
            name={this.props.name}
            onChange={(e) => {
              this.props.onChange(e, {
                ...this.props,
                value: e.target.value
              })
            }}
            disabled={(this.props.disabled)}
            value={this.props.value === undefined || this.props.value === null
                   ? ''
                   : '' + this.props.value}
            readOnly={(this.props.readonly)}
            type={this.props.type}
            onKeyPress={this.props.onKeyPress}
          />

          {
            this.props.postfixText &&
            <span className="material-padding">
              {
                this.props.postfixText
              }
            </span>
          }

          <span
            className={
              this.props.validations &&
              !this.props.validations.every((validation) => validation(this.props.value))
              ? 'bar error-bar'
              :
              'bar'
            }
          >
          </span>
        </Input>

        {
          this.props.validations &&
          <div className="error-colored brake-words">
            {
              !this.props.validations.every((validation) => validation(this.props.value)) &&
              this.props.validationMessageKey
            }
          </div>
        }

      </div>
    )
  }
}

const _default = connect(mapStateToProps, mapDispatchToProps)(MaterialInput)
export default _default
