/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";

import ListView from './listView'

import SinglePanelSiteWrapper from '../../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../../material/materialSearchInput'

import EnterReleaseCodeDialog from '../dialogs/enterReleaseCodeDialog'

import {
  getOpenExercisesViaVisibility,
  loadOpenExercisePreviewsViaVisibilityAsync,
  setActivePaginationPage, setOpenExercisesViaVisibility,
  setPaginationPageSize, setSearchText,
  setSortByKey
} from "../../../../state/actions/allAccessibleExercises/openExercisesSite/openExercisesSiteActions";
import Spinner from "../../../helpers/spinner";
import TagFilterPanel from '../../../tagComponents/tagsFilterPanel'
import {getI18n} from "../../../../../i18n/i18nRoot";
import {InputOnChangeEvent} from '../../../../types/reactEvents'
import {Moment} from 'moment'
import {getServerTime} from '../../../../communicationLayer/timeLayer'
import Logger from '../../../../helpers/logger'
import {updateExercisePreviews} from '../../../../helpers/editorHelper'
import moment from "moment";
import {warningDialog} from '../../../../helpers/dialogHelper'
import {ErrorHelper} from '../../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  /**
   * true: load data,
   * false: not
   * this is ued in the all closed exercises site because not both (via code, via visibility) should load data
   * and they use the same state...
   */
  readonly shouldLoadData: boolean

  /**
   * if we display both via code & via visibility we would get 2 dialogs...
   */
  readonly canDisplayEnterReleaseCodeDialog: boolean

  /**
   * true: can display, false not (probably because parent does)
   */
  readonly canDisplayTagFilterPanel: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    openExercisePreviewsViaVisibility: rootState.openExercisesState.openExercisePreviewsViaVisibility,
    isLoading: rootState.openExercisesState.isLoading
      || rootState.openExercisesState.isLoadingViaVisibility
      || rootState.manageTagsState.isLoading,
    isEnterReleaseCodeDialogDisplayed: rootState.openExercisesState.isEnterReleaseCodeDialogDisplayed,
    shouldLoadData: props.shouldLoadData,
    sortByKey: rootState.openExercisesState.paginationViaVisibility.orderByProp,
    sortDirection: rootState.openExercisesState.paginationViaVisibility.isOrderByDesc,
    pagination: rootState.openExercisesState.paginationViaVisibility,
    canDisplayEnterReleaseCodeDialog: props.canDisplayEnterReleaseCodeDialog,

    selectedFilterTagsIds: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,

    canDisplayTagFilterPanel: props.canDisplayTagFilterPanel,
    isTagsFilterDisplayed: rootState.openExercisesState.isTagsFilterDisplayed,
    tags: rootState.manageTagsState.tags,
    langId: rootState.i18nState.langId

  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadOpenExercisePreviewsViaVisibilityAsync,
                                                                        getOpenExercisesViaVisibility,
                                                                        setSortByKey,
                                                                        setPaginationPageSize,
                                                                        setActivePaginationPage,
                                                                        setSearchText,
                                                                        setOpenExercisesViaVisibility
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps

// noinspection TsLint
let updateLeftTimeIntervalsViaVisibility: number = -1

class OpenExercisesViaVisibilitySite extends React.Component<Props, any> {

  async componentWillMount(): Promise<void> {

    try {
      if (this.props.shouldLoadData) {
        await this.props.loadOpenExercisePreviewsViaVisibilityAsync()
      } else {
        await this.props.getOpenExercisesViaVisibility()
      }
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
      return //then do not setup timers...
    }

    await this.setupTimeIsUpIntervals()

  }

  componentWillUnmount(): void {
    clearInterval(updateLeftTimeIntervalsViaVisibility)
  }

  async setupTimeIsUpIntervals(): Promise<void> {

    // noinspection TsLint
    let serverTime: Moment = moment()

    try {
      serverTime = await getServerTime();
    } catch (err) {
      Logger.error('could not get server time')
      warningDialog('', getI18n(this.props.langId, "Could not get the server time, thus the displayed (left) times are probably wrong. The server is probably offline."), this.props.langId)
    }

    const clientServerTimeDiffInS = moment().diff(serverTime, 'seconds')

    clearInterval(updateLeftTimeIntervalsViaVisibility)

    updateLeftTimeIntervalsViaVisibility = setInterval(() => {

      const copy = updateExercisePreviews(this.props.openExercisePreviewsViaVisibility, clientServerTimeDiffInS, this.props.langId)

      this.props.setOpenExercisesViaVisibility(copy)

    }, 1000)
  }

  render(): JSX.Element {

    if (this.props.isEnterReleaseCodeDialogDisplayed && !this.props.canDisplayEnterReleaseCodeDialog) {
      //the other site (via code) displays the dialog already
      return null
    }


    return (

      <div className="multiple-single-panels">

        {
          (this.props.canDisplayTagFilterPanel && !this.props.isEnterReleaseCodeDialogDisplayed && this.props.isTagsFilterDisplayed) &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
            onTagsUpdated={this.props.getOpenExercisesViaVisibility}
          />
        }

        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          {
            !this.props.isEnterReleaseCodeDialogDisplayed &&
            <div className="view-padding">
              <div className="view-head-line">
                <div className="view-caption">
                  <h1>
                    {
                      getI18n(this.props.langId, 'Open exercises via visibility')
                    }
                  </h1>
                </div>
                <div className="view-options">
                  <MaterialSearchInput
                    value={this.props.pagination.searchText}
                    onChange={(e: InputOnChangeEvent) => {
                      this.props.setSearchText(e.currentTarget.value, false)
                    }}
                  />
                </div>
              </div>

              <div className="view-content">
                  <ListView
                    pagination={this.props.pagination}
                    setPaginationPageSize={this.props.setPaginationPageSize}
                    setActivePaginationPage={this.props.setActivePaginationPage}
                    exercisePreviews={this.props.openExercisePreviewsViaVisibility}
                    displayAsViaCode={false}
                    sortByKey={this.props.sortByKey}
                    sortDirection={this.props.sortDirection}
                    setSortByKey={this.props.setSortByKey}
                    hideShouldNotCountColumn={true}
                  />
              </div>
            </div>
          }

          {
            (!this.props.isLoading && this.props.isEnterReleaseCodeDialogDisplayed && this.props.canDisplayEnterReleaseCodeDialog) &&
            <EnterReleaseCodeDialog/>
          }


        </SinglePanelSiteWrapper>

      </div>

    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(OpenExercisesViaVisibilitySite)
