/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {Icon} from "semantic-ui-react";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {HelpPopup} from "../../../helpers/helpPopup";
import Logger from "../../../../helpers/logger";
import {isTestResultValid} from "../../../../helpers/testHelper";
import {RunnableCustomTestFrontendExclusive} from "../../../../types/customTests";
import {RunnableCustomProjectTestFrontendExclusive} from "../../../../types/customProjects";
import {getI18n} from "../../../../../i18n/i18nRoot";


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly customTests: ReadonlyArray<RunnableCustomTestFrontendExclusive | RunnableCustomProjectTestFrontendExclusive>

  readonly isSomeTestRunning: boolean
  readonly openCreateCustomTestDialog: () => void
  readonly canUserCreateCustomTests: boolean

  readonly runCustomTests: (tests: ReadonlyArray<RunnableCustomTestFrontendExclusive | RunnableCustomProjectTestFrontendExclusive>) => Promise<boolean>
  readonly resetCustomTestResults: () => void

  /**
   * e.g. used to save changes in the editor
   */
  readonly beforeRunningTests: () => Promise<void>
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    systemSettings: rootState.globalSystemSettingsState.systemSettings,
    maxNumberOfTestsWithOneRequest: rootState.globalSystemSettingsState.systemSettings
    && rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest >= 0
                                    ? rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest : 1,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class testsPanelHeader extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className={!this.props.isSomeTestRunning ? "flexed" : 'flexed div-disabled'}>

        <div className={`${this.props.canUserCreateCustomTests ? 'clickable' : 'flexed div-disabled' }`}
             onClick={() => {
               this.props.openCreateCustomTestDialog()
             }}
        >
          <Icon name="add"/>
          <span>
            {
              getI18n(this.props.langId,'Add')
            }
          </span>
        </div>

        <HelpPopup className="mar-left-half" wide defaultText={`${getI18n(this.props.langId, "The max amount of custom tests is")} ${this.props.systemSettings.maxCustomTestsPerParticipation < 0 ? getI18n(this.props.langId, "unlimited") : this.props.systemSettings.maxCustomTestsPerParticipation}`}/>

        <SimpleVDivider/>

        <div className={`mar-right-half ${(this.props.customTests.length > 0) ? 'clickable' : 'div-disabled' }`}
             onClick={async () => {

               //save changes if code changed
               await this.props.beforeRunningTests()

               const maxNumberOfTestsWithOneRequest = this.props.maxNumberOfTestsWithOneRequest

               Logger.debug("test panel header bar",
                 `running all test with ${maxNumberOfTestsWithOneRequest} test in parallel`)


               const validTests = this.props.customTests.filter(p => isTestResultValid(p))


               // noinspection TsLint
               let ranTests: RunnableCustomTestFrontendExclusive[] = []

               // noinspection TsLint
               for (let i = 0; i < this.props.customTests.length; i++) {
                 const test = this.props.customTests[i]

                 if (isTestResultValid(test) === false || validTests.length === this.props.customTests.length) {
                   ranTests.push(test)
                 } else {
                   //test was already valid
                 }


                 //we reached the limit
                 if (ranTests.length === maxNumberOfTestsWithOneRequest) {

                   const hasAnyTestFailed = await this.props.runCustomTests(ranTests)

                   //clear ran test
                   ranTests = []

                   //stop if at least 1 test failed
                   if (hasAnyTestFailed) {
                     //return instead of break else we would run the last check below which would be always true
                     return
                   }
                 }
               }

               //we might have some last tests but not reached the max num of tests with one request...
               if (ranTests.length > 0) {
                 //we missed the last < maxNumberOfTestsWithOneRequest tests
                 //these are still in the last tempCommand
                 await this.props.runCustomTests(ranTests)
               }


             }}
        >
          <Icon name="forward"/>
          <span>
            {
              getI18n(this.props.langId,'Run all tests')
            }
          </span>
        </div>

        <HelpPopup
          defaultText={
            `${getI18n(this.props.langId,'Runs all not passed tests. If all tests have passed then all tests are re-run. When at least one test fails then the next x tests are not executed.')} ${getI18n(this.props.langId,'Max parallel tests')}: ${this.props.maxNumberOfTestsWithOneRequest}`}
          wide
        />

        <SimpleVDivider/>

        <div className="mar-right-half clickable"
             onClick={() => {
               this.props.resetCustomTestResults()
             }}
        >
          <Icon name="remove"/>
          <span>
          {
            getI18n(this.props.langId, 'Clear test results')
          }
          </span>
        </div>
        <HelpPopup
          defaultText={getI18n(this.props.langId,'This only discards the results in the frontend, the results on the server are still there')}
          wide
        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(testsPanelHeader)
