/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {HelpPopup} from "../../../helpers/helpPopup";
import {AssessedSubmitTestFrontendExclusive} from "../../../../types/testResults";
import {percentageNeededToPassedAutomaticAssessment} from "../../../../constants";
import {isTestResultValid} from "../../../../helpers/testHelper";
import {RunTestCommandForBackend} from "../../../../types/testsAndTestProtocol";
import Logger from "../../../../helpers/logger";
import {runAfterSubmitTest} from "../../../../state/actions/doExerciseSite/doExerciseCrudActions";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {Icon} from "semantic-ui-react";
import {resetAfterSubmitTestResults} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {getI18n} from "../../../../../i18n/i18nRoot";
import {saveDoExerciseAfterSolutionAsync} from '../../../../state/actions/doExerciseSite/afterSolutionCrudActions'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly submitTestResults: ReadonlyArray<AssessedSubmitTestFrontendExclusive>
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    submitTestResults: props.submitTestResults,

    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    systemSettings: rootState.globalSystemSettingsState.systemSettings,

    afterEditorHasUnsavedChanges: rootState.doExerciseSiteState.afterEditorState.hasUnsavedChanges,

    //normally only check after test for running ... but better check all ;)
    isSomeTestRunning: rootState.doExerciseSiteState.exercise.tests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.exercise.customTests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.afterNormalTests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.afterSubmitTests.some(p => p.uiIsRunning)
    ,

    afterSolution: rootState.doExerciseSiteState.afterSolution,

    maxNumberOfTestsWithOneRequest: rootState.globalSystemSettingsState.systemSettings
                                    && rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest >= 0
                                    ? rootState.globalSystemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest
                                    : 1,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        runAfterSubmitTest,
                                                                        resetAfterSubmitTestResults,
                                                                        saveDoExerciseAfterSolutionAsync,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class submitTestsPanelHeaderBar extends React.Component<Props, any> {


  async runAllAfterSubmitTests(): Promise<void> {

    //save changes if code changed
    //this also ensures that the old/now invalid after saving test results are cleared
    if (this.props.afterEditorHasUnsavedChanges) {
      await this.props.saveDoExerciseAfterSolutionAsync(this.props.releaseCode,
                                                        this.props.afterSolution.pLangId,
                                                        true, //because editor has changed
                                                        this.props.afterSolution, this.props.langId
      )
    }

    const maxNumberOfTestsWithOneRequest = this.props.maxNumberOfTestsWithOneRequest

    Logger.debug("test panel header bar",
                 `running all test with ${maxNumberOfTestsWithOneRequest} test in parallel`
    )

    const validTests = this.props.submitTestResults.filter(p => isTestResultValid(p))

    //a template
    // noinspection TsLint
    let tempCommand: RunTestCommandForBackend = {
      generatedCode: this.props.releaseCode,
      isSubmitTest: true,
      testIds: [],
      isNormalTest: false,
      plangId: this.props.afterSolution.pLangId,
      isCustomTest: false
    }

    // noinspection TsLint
    for (let i = 0; i < this.props.submitTestResults.length; i++) {
      const test = this.props.submitTestResults[i]

      if (isTestResultValid(test) === false || validTests.length === this.props.submitTestResults.length) {
        //add
        tempCommand = {
          ...tempCommand,
          testIds: [...tempCommand.testIds, test.id]
        }
      } else {
        //test was already valid
      }

      //we reached the limit
      if (tempCommand.testIds.length === maxNumberOfTestsWithOneRequest) {
        const hasAnyTestFailed = await this.props.runAfterSubmitTest(tempCommand)

        //clear test ids
        tempCommand = {
          ...tempCommand,
          testIds: []
        }

        //stop if at least 1 test failed
        if (hasAnyTestFailed) {
          //return instead of break else we would run the last check below which would be always true
          return
        }
      }
    }

    //we might have some last tests but not reached the max num of tests with one request...
    if (tempCommand.testIds.length > 0) {
      //we missed the last < maxNumberOfTestsWithOneRequest tests
      //these are still in the last tempCommand
      await this.props.runAfterSubmitTest(tempCommand)
    }

  }

  // noinspection TsLint
  render(): JSX.Element {

    const maxSummedPoints = this.props.submitTestResults
      .map(p => p.weight)
      .reduce(((previousValue, currentValue) => previousValue + currentValue), 0)

    const reachedSummedPoints = this.props.submitTestResults
      .map(p => p.result && p.result.passed
                ? p.weight
                : 0)
      .reduce(((previousValue, currentValue) => previousValue + currentValue), 0)

    // noinspection TsLint
    const reachedPercentage = maxSummedPoints > 0
                              ? reachedSummedPoints * 100 / maxSummedPoints
                              : 100

    // noinspection TsLint
    const hasPassedSummed = reachedPercentage >= percentageNeededToPassedAutomaticAssessment

    return (
      <div className={!this.props.isSomeTestRunning
                      ? "flexed"
                      : 'flexed div-disabled'}>

        <div className={`flexed mar-right-half ${hasPassedSummed
                                                 ? 'ok-colored'
                                                 : 'error-colored'}`}>
          <div>
            {
              reachedSummedPoints
            }
          </div>
          /
          <div className="mar-right-half">
            {
              maxSummedPoints
            }
          </div>
          (
          <div>
            {
              reachedPercentage.toFixed(2)
            }
            %
          </div>
          )
        </div>

        <SimpleVDivider></SimpleVDivider>

        <div
          className={`mar-right-half ${this.props.submitTestResults.length > 0
                                       ? 'clickable'
                                       : 'div-disabled' }`}
          onClick={async () => {
            await this.runAllAfterSubmitTests()
          }}
        >
          <Icon name="forward"/>
          <span>
            {
              getI18n(this.props.langId, 'Run all tests')
            }
          </span>
        </div>

        <HelpPopup defaultText={`${getI18n(this.props.langId,
                                           'you need at least (>= '
        )}${percentageNeededToPassedAutomaticAssessment}${getI18n(this.props.langId,
                                                                  ') to pass the exercise. The weight (points) of the tests is displayed as the background of each test.'
        )}`}
                   wide
        />

        <SimpleVDivider/>

        <div className="clickable mar-right-half"
             onClick={() => {
               this.props.resetAfterSubmitTestResults()
             }}
        >
          <Icon name="remove"/>
          <span>
          {
            getI18n(this.props.langId, 'Clear test results')
          }
          </span>
        </div>
        <HelpPopup
          defaultText={getI18n(this.props.langId,
                               'This only discards the results in the frontend, the results on the server are still there'
          )}
          wide
        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(submitTestsPanelHeaderBar)