/**
 * Created by janis dähne.
 */

import * as React from "react";
import {CSSProperties} from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {CompareTestOptions, TestProtocolItem, TestProtocolType} from "../../../../types/testResults";
import {Checkbox, Divider, Icon, List} from "semantic-ui-react";
import {leftIconPopupOffset} from "../../../../constants";
import * as JsDiff from "diff";
import {startsWith} from "../../../../helpers/stringHelper";
import {HelpPopup} from "../../../helpers/helpPopup";
import {MyPopup} from "../../../helpers/myPopup";
import {getI18n} from "../../../../../i18n/i18nRoot";
import Logger from '../../../../helpers/logger'
import {CompareFileTestContentFrontendOnly} from '../../../../types/exerciseEditor'
import {convertCompareTestToObj, TestProtocol} from '../../../../types/testsAndTestProtocol'


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly item: TestProtocolItem
  /**
   * the next item to display a diff if output mismatched (or null if it's the last)
   */
  readonly nextItem: TestProtocolItem | null

  /**
   * the prev item to get infos
   */
  readonly prevItem: TestProtocolItem | null

  /**
   * true: use js diff to display the diff between the expected output (unchanged) and the real output
   * (instead of the real output the diff is shown)
   * false: if output is wrong only mark it (e.g. red) but output real output
   */
  readonly useDiffIfOutputMismatched: boolean

  /**
   * true: replace all whitespaces by bullet signs
   */
  readonly useReplaceWhiteSpacesWithBullets: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


// noinspection TsLint
/**
 * composes x funcs into one new function ... the functions are applied sequentially
 * @param {(line: string) => string} funcs
 * @returns {(line: string) => string}
 */
function compositeFuncs(...funcs: Array<(line: string) => string>): ((line: string) => string) {

  //function composition f(g(x))  in haskell we would write return (f . g)
  return (line) => {
    // noinspection TsLint
    let result = line
    for (const func of funcs) {
      result = func(result)
    }
    return result
  }
}


interface AsciiColorCodeMap {
  readonly [code: string]: CSSProperties
}

/**
 * see https://en.wikipedia.org/wiki/ANSI_escape_code#Escape_sequences ... we use ubuntu colors
 * backend (test runner) makes sure \033 or \u001B (both esc) are translated to \\033 for frontend
 */
const asciiColorCodeMap: AsciiColorCodeMap = {

  /**
   * resets
   */
  '\\033[0m': null,

  '\\033[1m': {
    fontWeight: 'bold'
  },
  '\\033[4m': {
    textDecoration: 'underline'
  },
  /**
   * this only works if one used a color/bg color
   * using only this won't work because we have no bg color by default... (will only invert foreground text color)
   */
  '\\033[7m': {
    filter: 'invert(100%)'
  },

  //fg color
  '\\033[30m': {
    color: 'rgb(1,1,1)'
  },
  '\\033[31m': {
    color: 'rgb(222,56,43)'
  },
  '\\033[32m': {
    color: 'rgb(57,181,74)'
  },
  '\\033[33m': {
    color: 'rgb(255,199,6)'
  },
  '\\033[34m': {
    color: 'rgb(0,111,184)'
  },
  '\\033[35m': {
    color: 'rgb(118,38,113)'
  },
  '\\033[36m': {
    color: 'rgb(44,181,233)'
  },
  '\\033[37m': {
    color: 'rgb(204,204,204)'
  },
  //fg color bright
  '\\033[90m': {
    color: 'rgb(\t128,128,128)'
  },
  '\\033[91m': {
    color: 'rgb(255,0,0)'
  },
  '\\033[92m': {
    color: 'rgb(0,255,0)'
  },
  '\\033[93m': {
    color: 'rgb(255,255,0)'
  },
  '\\033[94m': {
    color: 'rgb(0,0,255)'
  },
  '\\033[95m': {
    color: 'rgb(255,0,255)'
  },
  '\\033[96m': {
    color: 'rgb(0,255,255)'
  },
  '\\033[97m': {
    color: 'rgb(255,255,255)'
  },

  //bg color
  '\\033[40m': {
    backgroundColor: 'rgb(1,1,1)'
  },
  '\\033[41m': {
    backgroundColor: 'rgb(222,56,43)'
  },
  '\\033[42m': {
    backgroundColor: 'rgb(57,181,74)'
  },
  '\\033[43m': {
    backgroundColor: 'rgb(255,199,6)'
  },
  '\\033[44m': {
    backgroundColor: 'rgb(0,111,184)'
  },
  '\\033[45m': {
    backgroundColor: 'rgb(118,38,113)'
  },
  '\\033[46m': {
    backgroundColor: 'rgb(44,181,233)'
  },
  '\\033[47m': {
    backgroundColor: 'rgb(204,204,204)'
  },

  //bg color bright
  '\\033[100m': {
    color: 'rgb(\t128,128,128)'
  },
  '\\033[101m': {
    color: 'rgb(255,0,0)'
  },
  '\\033[102m': {
    color: 'rgb(0,255,0)'
  },
  '\\033[103m': {
    color: 'rgb(255,255,0)'
  },
  '\\033[104m': {
    color: 'rgb(0,0,255)'
  },
  '\\033[105m': {
    color: 'rgb(255,0,255)'
  },
  '\\033[106m': {
    color: 'rgb(0,255,255)'
  },
  '\\033[107m': {
    color: 'rgb(255,255,255)'
  },
}

const allAsciiColorCodes = Object.keys(asciiColorCodeMap)


class consoleItem extends React.Component<Props, any> {


  convertConsoleLineToAsciiColoredLine(line: string): ReadonlyArray<JSX.Element> | string {

    if (line.includes('\\0') === false) {
      return line
    }

    // tslint:disable-next-line
    let parts: JSX.Element[] = []


    //we can't use a default style because output has already a background color so we can't use white...

    // tslint:disable-next-line
    let currIndex = 0, lastIndex = 0


    // tslint:disable-next-line
    let currentStyle: CSSProperties = {}

    while (currIndex < line.length) {

      const part = line.substring(lastIndex, currIndex)

      // tslint:disable-next-line
      for (let i = 0; i < allAsciiColorCodes.length; i++) {
        const colorCode = allAsciiColorCodes[i]

        //...RED
        if (part.endsWith(colorCode)) {
          //add ... without RED

          if (part.length > 0) {
            parts.push(<span key={currIndex} style={currentStyle}>{part.substr(0,
                                                                               part.length - colorCode.length
            )}</span>)
          } else {
            //we could have >1 color codes after another ... so we only modify the style
          }


          if (asciiColorCodeMap[colorCode] === null) {
            //reset
            currentStyle = {}
          } else {
            //add the color code style to the current style
            currentStyle = {
              ...currentStyle,
              ...asciiColorCodeMap[colorCode]
            }
          }

          lastIndex = currIndex
        }
      }
      currIndex++
    }
    const lastPart = line.substring(lastIndex, currIndex)

    if (lastPart === '') {
      //ok we had an color code at the end
    } else {
      //some normal text at the end
      parts.push(<span key={currIndex} style={currentStyle}>{lastPart}</span>)
    }

    return parts
  }

  render(): JSX.Element {


    if (this.props.item.type === TestProtocolType.isExpectedExitCode) return null

    const nextItemContent = this.props.nextItem
                            ? this.props.nextItem.content
                            : ''

    // noinspection TsLint
    let contentTransformer: (line: string) => string = line => line

    if (this.props.useReplaceWhiteSpacesWithBullets) {

      //contentTransformer = (line) => line.replace(/\s/g, "·") //matches any space...

      contentTransformer =
        compositeFuncs(
          //replace all spaces with ·
          (line) => line.replace(/ /g, "·"),
          //replace all tabs with ⇥ (mac tab sign)
          (line) => line.replace(/\t/g, "⇥")
        )
    }

    // tslint:disable-next-line
    let compareTest: CompareFileTestContentFrontendOnly | null = null

    if (this.props.item.type === TestProtocolType.isCompareTestCompare) {
      compareTest = convertCompareTestToObj(this.props.item.content)
    }

    // tslint:disable-next-line
    let compareTestActual: {lineNr: number, line: string} | null

    if (this.props.item.type === TestProtocolType.isCompareTestResultActual) {
      //is: cactual [line nr] [line]
      const line = this.props.item.content.substring(TestProtocol.compareOutputFileTestOutputActual.length + 1)

      // tslint:disable-next-line
      let lineNr = 0

      // tslint:disable-next-line
      let lineNrString = ''

      // tslint:disable-next-line
      for (let i = 0; i < line.length; i++) {
        const c = line[i]

        if (c === ' ') break

        lineNrString += c
      }

      lineNr = parseInt(lineNrString, 10)

      if (isNaN(lineNr)) {
        Logger.warn(`could not convert ${lineNrString} in compare test item to line number`)
        lineNr = -2
      }

      const lineContent = line.substring(lineNrString.length + 1)

      compareTestActual = {
        lineNr: lineNr,
        line: lineContent
      }
    }

    // tslint:disable-next-line
    let compareTestExpected: {lineNr: number, line: string} | null

    if (this.props.item.type === TestProtocolType.isCompareTestResultExpected
      || this.props.item.type === TestProtocolType.isCompareTestResultActual) { //for js diff we need to parse the next item content too

      //is: cactual [line nr] [line]
      const line = (this.props.item.type === TestProtocolType.isCompareTestResultExpected
                    ? this.props.item.content
                    : nextItemContent)
        .substring(TestProtocol.compareOutputFileTestOutputExpected.length + 1)

      // tslint:disable-next-line
      let lineNr = 0

      // tslint:disable-next-line
      let lineNrString = ''

      // tslint:disable-next-line
      for (let i = 0; i < line.length; i++) {
        const c = line[i]

        if (c === ' ') break

        lineNrString += c
      }

      lineNr = parseInt(lineNrString, 10)

      if (isNaN(lineNr)) {
        Logger.warn(`could not convert ${lineNrString} in compare test item to line number`)
        lineNr = -2
      }

      const lineContent = line.substring(lineNrString.length + 1)

      compareTestExpected = {
        lineNr: lineNr,
        line: lineContent
      }

    }

    // noinspection TsLint
    return (
      <List.Item style={{padding: 0}}>

        {
          this.props.item.type === TestProtocolType.isProgramArguments &&
          <div className="flexed console-arguments-line">

            <HelpPopup className="mar-right-half"
                       defaultText={getI18n(this.props.langId,
                                            'The console arguments for the program. Separated by whitespaces.'
                       )}/>

            <div className="mar-right-half">
              {getI18n(this.props.langId, 'Arguments')}:
            </div>
            <div>
              <pre className="mar-null">
                {
                  contentTransformer(this.props.item.content)
                }
              </pre>
            </div>
          </div>
        }
        {
          this.props.item.type === TestProtocolType.isError &&
          <div className="error-colored padded">
            <pre className="mar-null console-item-pre">
              {
                //don't transform errors
                this.props.item.content
              }
            </pre>
          </div>
        }
        {
          this.props.item.type === TestProtocolType.unknown &&
          <div className="grey-colored padded">
            <pre className="mar-null console-item-pre">
              {
                `(${getI18n(this.props.langId, 'unknown/ignored line')}):${contentTransformer(this.props.item.content)}`
              }
            </pre>
          </div>
        }
        {
          this.props.item.type === TestProtocolType.plain &&
          <div className="padded">
            <pre className="mar-null console-item-pre">
              {
                contentTransformer(this.props.item.content)
              }
            </pre>
          </div>
        }
        {
          //input
          this.props.item.type === TestProtocolType.isInput &&
          <div className="console-input-output-line">
            <div className="input-part light-grey-bg-colored">
              <div>
                <pre className="mar-null">
                {
                  contentTransformer(this.props.item.content)
                }
              </pre>
              </div>
            </div>
            <div className="program-part">

            </div>
            <div className="output-part">

            </div>
          </div>
        }
        {
          this.props.item.type === TestProtocolType.isMissingOutput &&
          <div className="console-input-output-line">
            <div className="input-part">
            </div>
            <div className={'program-part-mismatch'}>
            </div>
            <div className="output-part bg-white-colored">
              <div>
                <pre className="mar-null not-important-colored" style={{fontStyle: 'italic'}}>
                  missing
                  <span className="white-colored" style={{pointerEvents: 'none'}}>no.</span>
              </pre>
              </div>
            </div>
          </div>
        }
        {
          this.props.item.type === TestProtocolType.isOutput &&
          <div className="console-input-output-line">
            <div className="input-part">
            </div>
            <div className={this.props.item.hasOutputMismatched
                            ? 'program-part-mismatch'
                            : 'program-part'}>

            </div>
            <div className="output-part light-grey-bg-colored">

              {
                //all ok
                !this.props.item.hasOutputMismatched &&
                <div>
                  <pre className="mar-null">
                  {
                    startsWith(this.props.item.content, "(regex):") &&
                    <MyPopup trigger={
                      <Icon className="hoverable" name="registered"/>
                    }
                             content={getI18n(this.props.langId, "Your output must match this regex")}
                             position="top left"
                             offset={leftIconPopupOffset + 5}
                             wide
                    />
                  }
                  {
                    //only convert color if all is ok... else we have an mismatch and we want to compare...no colors
                    startsWith(this.props.item.content, "(regex):")
                    ? this.convertConsoleLineToAsciiColoredLine(contentTransformer(this.props.item.content.substr("(regex):".length)))
                    : this.convertConsoleLineToAsciiColoredLine(contentTransformer(this.props.item.content))
                  }
                  </pre>
                </div>
              }

              {
                (this.props.item.hasOutputMismatched
                  &&
                  (
                    !this.props.useDiffIfOutputMismatched
                    || (this.props.nextItem !== null && startsWith(this.props.nextItem.content, "(regex):")) //if next item is an expected regex then do output without showing diff... (other branch of see bot regex case)
                  )
                ) &&
                <div className="error-colored">
                  <pre className="mar-null">
                  {
                    contentTransformer(this.props.item.content)
                  }
                  </pre>
                </div>
              }
              {
                //diff output, if we use diff output we cannot use regex because this doesn't make sense
                (this.props.item.hasOutputMismatched && this.props.useDiffIfOutputMismatched
                  && (this.props.nextItem === null || !startsWith(this.props.nextItem.content, "(regex):")) //if next item is an expected regex then do not show diff (because not possible for regex)
                ) &&
                <div>
                  <pre className="mar-null">
                    {
                      JsDiff.diffChars(contentTransformer(nextItemContent),
                                       contentTransformer(this.props.item.content)
                      ).map((part, index) => {
                        return (
                          <span key={index}>
                          {
                            (!part.added && !part.removed) &&
                            <span>{part.value}</span>
                          }
                            {
                              (part.added && !part.removed) &&
                              <span className="error-colored">{part.value}</span>
                            }
                            {
                              (!part.added && part.removed) &&
                              <span className="diff-blue-colored">{part.value}</span>
                            }
                       </span>

                        )
                      })
                    }
                  </pre>
                </div>
              }

            </div>
          </div>
        }
        {
          //expected output
          this.props.item.type === TestProtocolType.isExpectedOutput &&
          <div className="console-input-output-line">
            <div className="input-part">

            </div>

            {
              this.props.useDiffIfOutputMismatched &&
              <MyPopup
                trigger={
                  <div className="clickable program-part-expected-output">
                  </div>
                }
                content={
                  getI18n(this.props.langId,
                          'The previous (your) output was wrong. This is the correct output. Blue characters are missing, red ones are too much (inside your output) [For regex lines, starting with \'(regex):\' there is no diff display option].'
                  )
                }
                position="top left"
                offset={leftIconPopupOffset + 5}
                wide
              />
            }

            {
              !this.props.useDiffIfOutputMismatched &&
              <MyPopup
                trigger={
                  <div className="clickable program-part-expected-output">
                  </div>
                }
                content={getI18n(this.props.langId,
                                 'The previous (your) output was wrong. This is the correct output.'
                )}
                position="top left"
                offset={leftIconPopupOffset + 5}
                wide
              />
            }


            <div className="expected-output-part">
              <div>
                <pre className="mar-null">
                  {
                    startsWith(this.props.item.content, "(regex):") &&
                    <MyPopup trigger={
                      <Icon className="hoverable" name="registered"/>
                    }
                             content={getI18n(this.props.langId, "Your output must match this regex")}
                             position="top left"
                             offset={leftIconPopupOffset + 5}
                             wide
                    />
                  }

                  {
                    startsWith(this.props.item.content, "(regex):")
                    ? contentTransformer(this.props.item.content.substr("(regex):".length))
                    : contentTransformer(this.props.item.content)
                  }
                </pre>
              </div>
            </div>
          </div>
        }

        {
          this.props.item.type === TestProtocolType.isEndOfTestLine &&
          <div>
            <Divider horizontal>
              {
                getI18n(this.props.langId, 'End of user program')
              }
            </Divider>
          </div>
        }

        {
          this.props.item.type === TestProtocolType.isCompareTestCompare &&
          <div className="console-input-output-line">
            <div className="padded">
                <span>
                  {getI18n(this.props.langId, "Comparing files")}&nbsp;
                </span>
              {
                compareTest === null &&
                <span>{getI18n(this.props.langId, "Parse error")}</span>
              }

              {
                compareTest !== null &&
                <div className="inline-div">
                  <span>'
                    {
                      compareTest.file1NameWithExtension
                    }
                    '</span>
                  &nbsp;
                  <span>
                    <Icon name="long arrow alternate right"/>
                  </span>
                  &nbsp;
                  <span>'
                    {
                      compareTest.file2NameWithExtension
                    }
                    '</span>

                  {
                    (compareTest.ignoreCase || compareTest.ignoreAllEmptyOrWhitespacesOnlyLines || compareTest.ignoreLeadingWhitespaces || compareTest.ignoreTrailingWhitespaces) &&
                    <div className="inline-div">
                      &nbsp;
                      <span>
                        {getI18n(this.props.langId, "with options")}
                      </span>
                      <MyPopup
                        trigger={
                          <span>
                        <Icon className="hoverable mar-left-half" name="question circle"/>
                      </span>
                        }
                        horizontalOffset={leftIconPopupOffset}
                        content={
                          <div className="flexed-h stacked-checkboxes">
                            <Checkbox
                              label={`${getI18n(this.props.langId, "Ignore case")} (${CompareTestOptions.ignoreCase})`}
                              checked={compareTest.ignoreCase}
                              disabled
                            />
                            <Checkbox
                              label={`${getI18n(this.props.langId,
                                                "Ignore all additional empty lines in the user file"
                              )} (${CompareTestOptions.ignoreAllEmptyOrWhitespacesOnlyLines})`}
                              checked={compareTest.ignoreAllEmptyOrWhitespacesOnlyLines}
                              disabled
                            />
                            <Checkbox label={`${getI18n(this.props.langId,
                                                        "Ignore leading whitespaces"
                            )} (${CompareTestOptions.ignoreLeadingWhitespaces})`}
                                      checked={compareTest.ignoreLeadingWhitespaces}
                                      disabled
                            />
                            <Checkbox label={`${getI18n(this.props.langId,
                                                        "Ignore trailing whitespaces"
                            )} (${CompareTestOptions.ignoreTrailingWhitespaces})`}
                                      checked={compareTest.ignoreTrailingWhitespaces}
                                      disabled
                            />
                          </div>
                        }
                      />
                    </div>
                  }

                </div>
              }


            </div>
          </div>
        }

        {
          this.props.item.type === TestProtocolType.isCompareTestResultOk &&
          <div className="console-input-output-line">
            <div className="padded">
            <span className="ok-colored">
              {getI18n(this.props.langId, "Equal")}
            </span>
            </div>
          </div>
        }

        {
          this.props.item.type === TestProtocolType.isCompareTestResultFile1NotFound &&
          <div className="console-input-output-line">
            <div className="padded error-colored">
            <span>
              {getI18n(this.props.langId, "File 1 was not found")}
            </span>
            </div>
          </div>
        }

        {
          this.props.item.type === TestProtocolType.isCompareTestResultFile2NotFound &&
          <div className="console-input-output-line">
            <div className="padded error-colored">
            <span>
              {getI18n(this.props.langId, "File 2 was not found")}
            </span>
            </div>
          </div>
        }

        {
          this.props.item.type === TestProtocolType.isCompareTestResultFile2EOF &&
          <div className="console-input-output-line">
            <div className="padded error-colored">
            <span>
              {getI18n(this.props.langId, "End of file 2 reached")}
            </span>
            </div>
          </div>
        }

        {
          this.props.item.type === TestProtocolType.isCompareTestResultActual &&
          <div className="console-input-output-line">

            <div className="program-part-mismatch">

            </div>

            <div className={this.props.useDiffIfOutputMismatched
                            ? 'output-part'
                            : 'output-part error-colored'
            }>
              <div>

                  <pre className="mar-null">
                    {getI18n(this.props.langId, "Line")}&nbsp;
                    {
                      compareTestActual.lineNr
                    }
                    :
                    {
                      compareTestActual.line === TestProtocol.compareOutputFileTestOutputFile2EOF
                      ? getI18n(this.props.langId, "END OF FILE")
                      : this.props.useDiffIfOutputMismatched === false
                        ? contentTransformer(compareTestActual.line)
                        : JsDiff.diffChars(contentTransformer(compareTestExpected.line),
                                           contentTransformer(compareTestActual.line)
                        ).map((part, index) => {
                          return (
                            <span key={index}>
                          {
                            (!part.added && !part.removed) &&
                            <span>{part.value}</span>
                          }
                              {
                                (part.added && !part.removed) &&
                                <span className="error-colored">{part.value}</span>
                              }
                              {
                                (!part.added && part.removed) &&
                                <span className="diff-blue-colored">{part.value}</span>
                              }
                       </span>

                          )
                        })

                    }
                    </pre>

              </div>
            </div>

          </div>
        }

        {
          this.props.item.type === TestProtocolType.isCompareTestResultExpected &&
          <div className="console-input-output-line">

            {
              this.props.useDiffIfOutputMismatched &&
              <MyPopup
                trigger={
                  <div className="clickable program-part-expected-output">
                  </div>
                }
                content={
                  getI18n(this.props.langId,
                          'The previous (your) output was wrong. This is the correct output. Blue characters are missing, red ones are too much (inside your output) [For regex lines, starting with \'(regex):\' there is no diff display option].'
                  )
                }
                position="top left"
                offset={leftIconPopupOffset + 5}
                wide
              />
            }

            {
              !this.props.useDiffIfOutputMismatched &&
              <MyPopup
                trigger={
                  <div className="clickable program-part-expected-output">
                  </div>
                }
                content={getI18n(this.props.langId,
                                 'The previous (your) output was wrong. This is the correct output.'
                )}
                position="top left"
                offset={leftIconPopupOffset + 5}
                wide
              />
            }

            <div className="expected-output-part">
              <div>
                <pre className="mar-null">
                  {getI18n(this.props.langId, "Line")}&nbsp;
                  {
                    compareTestExpected.lineNr
                  }
                  :
                  {
                    contentTransformer(compareTestExpected.line)
                  }
                </pre>
              </div>
            </div>

          </div>
        }

      </List.Item>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(consoleItem)
