/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {RunnableExerciseTestFrontendExclusive} from "../../../../types/exerciseEditor";
import {List, Icon} from "semantic-ui-react";
import TestNumber from './testNumber'
import orderBy from 'lodash-es/orderBy'
import {RunnableCustomTestFrontendExclusive} from "../../../../types/customTests";
import {AssessedSubmitTestFrontendExclusive} from "../../../../types/testResults";
import WeightIndicatorBackgroundBar from './weightIndicatorBackgroundBar'


import {getI18n} from "../../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  /**
   * true: display a background progress bar displaying the weight of the test if the test passes its green, if not red
   * false: not display any background progress bar
   */
  readonly displayWeightIndicatorsForResults: boolean
  /**
   * true: this panel displays after tests, false: not
   */
  readonly areAfterTests: boolean
  /**
   * the list of tests to display
   */
  readonly tests: ReadonlyArray<RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive>
  /**
   * the selected test to display inside the console
   * we need to know this to display an indicator so that the user sees fast which test was selected
   */
  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null

  /**
   * true: the editor has some unsaved changes ... save (call onSaveSolution) to save changes before testing
   * false: not
   */
  readonly editorHasUnsavedChanges: boolean

  /**
   * saves editor changes before running tests (checks editorHasUnsavedChanges)  (or null if user cannot run tests)
   * only null if editor content never changes...
   *
   * MAKE SURE to await the save else the isLoading won't be displayed!!
   */
  readonly onSaveSolution: () => Promise<void> | null

  /**
   * true: some test is running --> user can only run 1 test (or 1 test session) at one time
   * false: not
   */
  readonly isSomeTestRunning: boolean

  /**
   * true: user is basically allowed to run tests
   * false: not (e.g. if the release ended)
   */
  readonly isUserAllowedToRunTests: boolean

  /**
   * sets a test result to display in a console view
   */
  readonly setSelectedConsoleViewTest: (selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive
    | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => void

  /**
   * to run a test  (or null if user cannot run tests)
   *
   * make sure to set he test to running
   */
  readonly onRunTest: (test: RunnableExerciseTestFrontendExclusive
    | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => void | null

  /**
   * used to open a test view dialog
   */
  readonly openViewTestDialog: (runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    tests: props.tests,
    selectedConsoleViewTest: props.selectedConsoleViewTest,
    systemSettings: rootState.globalSystemSettingsState,
    editorHasUnsavedChanges: props.editorHasUnsavedChanges,
    isSomeTestRunning: props.isSomeTestRunning,
    isUserAllowedToRunTests: props.isUserAllowedToRunTests,
    areAfterTests: props.areAfterTests,

    onSaveSolution: props.onSaveSolution,

    setSelectedConsoleViewTest: props.setSelectedConsoleViewTest,
    onRunTest: props.onRunTest,
    openViewTestDialog: props.openViewTestDialog,

    displayWeightIndicatorsForResults: props.displayWeightIndicatorsForResults,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

const indicatorNotPassedBgColor = `rgba(221,81,76,0.1)` //error color alpha 0.1
const indicatorPassedBgColor = `rgba(94, 185, 94, 0.1)` //ok color alpha 0.1

class TestsPanel extends React.Component<Props, any> {
  render(): JSX.Element {

    const maxSummedPoints = this.props.tests
      .map(p => p.weight)
      .reduce(((previousValue, currentValue) => previousValue + currentValue), 0)

    return (
      <div className="fh">

        <List selection divided relaxed>
          {
            orderBy<RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive>(this.props.tests,
              (test) => test.displayIndex)
              .map((test, index) => {
                return (
                  //style hidden is needed because the test running animation (.test-running) rotates and box heights gets bigger... after 45 deg
                  <List.Item key={test.id} style={{overflow: 'hidden'}}
                             className={`test-line pad-left-right default-cursor fg-colored indicator-parent ` +
                             (this.props.selectedConsoleViewTest
                               && this.props.selectedConsoleViewTest.id === test.id
                              && this.props.areAfterTests === this.props.selectedConsoleViewTest.uiIsAfterTest
                                 ? 'line-active' : ''
                             )}>

                    {
                      this.props.displayWeightIndicatorsForResults &&
                      <WeightIndicatorBackgroundBar value={test.weight}
                                                    maxValue={maxSummedPoints}
                                                    indicatorColor={
                                                      test.result && test.result.passed ? indicatorPassedBgColor : indicatorNotPassedBgColor
                                                    }
                      />
                    }

                    <List.Content className="indicator-sibling">
                      <div className="flexed">
                        <TestNumber number={(index + 1)} isValid={test.result ? test.result.passed : null}
                                    isTestRunning={test.uiIsRunning}
                                    className="mar-right"
                                    isEnabled={!this.props.isSomeTestRunning && this.props.isUserAllowedToRunTests}
                                    onClick={async () => {

                                      //save changes if code changed
                                      //this also ensures that the old/now invalid after saving test results are cleared
                                      if (this.props.editorHasUnsavedChanges && this.props.onSaveSolution) {
                                        await this.props.onSaveSolution()
                                      }

                                      this.props.onRunTest(test)
                                      // const command: RunTestCommandForBackend = {
                                      //   generatedCode: this.props.releaseCode,
                                      //   plangId: this.props.solutionPLangId,
                                      //   testIds: [test.id],
                                      //   isNormalTest: true,
                                      //   isCustomTest: false,
                                      //   isSubmitTest: false,
                                      //   solutionVersionId: null
                                      // }
                                      //
                                      // this.props.runDoExerciseTest(command)

                                    }}
                        />

                        <div className="flexed v-centered flex-stretch">
                          <div>
                            {
                              test.displayName
                            }
                          </div>

                          {
                            test.result &&
                            <div className="clickable mar-left v-centered"
                                 onClick={() => {
                                   this.props.setSelectedConsoleViewTest(test)
                                 }}
                            >
                              <Icon name="info circle"/>
                              <span>
                                {
                                  getI18n(this.props.langId,'Result')
                                }
                              </span>
                            </div>
                          }
                        </div>

                        <div className="clickable v-centered"
                             onClick={() => {
                               this.props.openViewTestDialog(test)
                             }}
                        >
                          <Icon name="info circle"/>
                          <span>
                            {
                              getI18n(this.props.langId, 'Show content')
                            }
                          </span>
                        </div>


                      </div>
                    </List.Content>
                  </List.Item>
                )
              })
          }

        </List>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TestsPanel)
