/**
 * Created by janis dähne.
 */
import {debugOriginUrl} from "../debug";
import {AssessmentFullBase} from "./types/submissions";
import {PaginationPostData, PaginationStateData} from './types/pagination'
import Logger from './helpers/logger'


/**
 * x.y.z
 * x - big updates
 * y - breaking changes / new features
 * z - fixes, small changes
 */
export const versionString = '2.6.2'


export const supportMail = 'yapex@informatik.uni-halle.de'

/**
 * the default mode for the editor when we got an invalid mode
 */
export const defaultEditorMode = "text"


/**
 * we use a div for fronting where we insert the html to print
 */
export const tempPrintDivId = 'temp-print-div-id'

declare var safari: any
/**
 * we need this for printing... safari will not emit onafterprint
 */
export function browser_isSafari(): boolean {
  //see https://stackoverflow.com/questions/9847580/how-to-detect-safari-chrome-ie-firefox-and-opera-browser/9851769

  // // Firefox 1.0+
  // var isFirefox = typeof InstallTrigger !== 'undefined';

  // Safari 3.0+ "[object HTMLElementConstructor]"
  // @ts-ignore
  const isSafari = /constructor/i.test(window.HTMLElement) || (function (p: any): any {
    return p.toString() === "[object SafariRemoteNotification]";
  })(!window['safari'] || (typeof safari !== 'undefined' && safari.pushNotification));

  return isSafari
}


/**
 * when we create files in the editor view this is used to validate file names... should be the same as in backend check
 * no spaces because they are bad for paths...
 */
export const validFilenameRegex = /^([a-z0-9_.-]+\.[^.]+)|^([a-z0-9_-]+)$/i

/**
 * the interval we get the server message
 */
export const refreshServerMessageIntervalInS = 150 //2,5 min


/**
 * when the automatic refresh toggle button is checked we use this interval
 */
export const automaticRefreshSubmissionsListViewIntervalInS = 6

export const defaultTaskDescriptionFontSizeInPx = 16

/**
 * actually this is a cookie
 */
export const crsfCookieName = "CSRF-Token";
export const crsfReturnHeaderName = "X-CSRF-Token";

//from https://stackoverflow.com/questions/10730362/get-cookie-by-name
export function getCookie(name: string): string | null {
  const match = document.cookie.match(new RegExp('(^| )' + name + '=([^;]+)'));
  if (match) return match[2];

  return null
}

// noinspection TsLint
export let currentCrsfToken: string | null = null

export function setCurrentCrsfToken(token: string): void {
  currentCrsfToken = token
  // Logger.log('found crsf token, setting header')
}

export const copyrightPageLinkPath = '/copyright/copyright.html' //see webpack.config.dist.js -> CopyWebpackPlugin
export const privacyPolicyPageLinkPath = '/privacypolicy/privacy.html' //see webpack.config.dist.js -> CopyWebpackPlugin
export const aboutPageLinkPath = '/about/about.html'


/**
 * returns the download personal data link for the given user (id)
 * @param userId null: to download own personal data, id (>0) to download other users data (requires specific system role permissions)
 */
export function getPersonalDataDownloadLinkPart(userId: number | null): string {

  if (userId === null) {
    return `${debugOriginUrl}/api/users/privacy/download`
  }

  return `${debugOriginUrl}/api/users/privacy/download/${userId}`
}

// noinspection TsLint
/**
 * the interval when editing with some editor (currently only do exercise editors)
 * and the last known content was changed then update auth token lifetime so we keep
 * logged in
 *
 * even when we get false negatives (e.g. we changed the selected file) this is ok
 * we only want to catch cases where the user is idle for a long time...
 */
export const refreshAuthTokenLifetimeIntervalWhenEditingInMs = 4 * 60 * 1000 //use 4 because backend uses 30 mins with 5 we would probably hit this somehow

/**
 * the time we wait until we emit an onBlur event to update the redux store from the ace storage
 */
export const multiFileEditorDebounceInMs = 1500

export const descriptionEditorDebounceInMs = 500
export const tutorFeedbackEditorDebounceInMs = 500

export const paginationPageSizeOptionAll = -1
// noinspection TsLint
export const paginationPageSizeOptions = [10, 15, 25, 50, 100, 200, paginationPageSizeOptionAll] //-1 is all

export function getInitialPaginationData<T>(): PaginationStateData<T> {
  return {
    searchText: '',
    currentPage: 1, //starts with 1
    isOrderByDesc: false, //ascending
    orderByProp: null,
    pageSize: paginationPageSizeOptions[0],
    totalPages: 0,
    totalItems: 0
  }
}

/**
 * this should returns all possible items not paged
 */
export const getAllPaginationItemsPostData: PaginationPostData = {
  orderByProp: null,
  isOrderByDesc: null,
  searchText: '',
  pageSize: paginationPageSizeOptionAll, //get all
  page: 1
}

/**
 * usage:
 *  const debouncedRefresh = debounce((dispatch) => {
 *   dispatch(refreshPagePaginatedData())
 * }, searchInputDebounceInMs)
 *
 *  export function setSearchText(searchText: string): MultiActions {
 *   return (dispatch, getState) => {
 *     dispatch(_setSearchText(searchText))
 *
 *     debouncedRefresh(dispatch)
 *   }
 * }
 */
export const searchInputDebounceInMs = 800

/**
 * the percentage needed to pass the automatic assessment
 * a message will be displayed to indicate if the user has passed
 *
 * the comparison will be made as
 *
 * user reached points >= percentageNeededToPassedAutomaticAssessment ? --> passed
 *
 * @type {number}
 */
export const percentageNeededToPassedAutomaticAssessment = 100 //TODO maybe in system settings??

export const homeLinkPath = '/'

export const systemSettingsSiteLinkPath = '/system/settings'

export const dashboardLinkPath = '/dashboard'

export const ownExercisesLinkPath = '/exercises/own'
export const groupExercisesLinkPath = '/exercises/group'

//displays both via code and via visibility
export const openExercisesLinkPath = '/exercises/open/all'
//displays both via code and via visibility
export const closedExercisesLinkPath = '/exercises/closed/all'

//only the releases where the user entered a code
export const openExercisesViaCodeLinkPath = '/exercises/open/code'
//via release setting is visible for all
export const openExercisesViaVisibilityLinkPath = '/exercises/open/visible'

//only the releases where the user entered a code
export const closedExercisesViaCodeLinkPath = '/exercises/closed/code'
//via release setting is visible for all
export const closedExercisesViaVisibilityLinkPath = '/exercises/closed/visible'


export const groupsLinkPath = '/groups'


export const markdownSiteTestSyntaxGuide = '/guides/markdown/tests'
export const markdownSiteExerciseSyntaxGuide = '/guides/markdown/exercise'

/**
 * only for the router ... for programmatic access use getExerciseReleaseLinkPath
 * this is used in the route component
 * @type {string}
 */
export const exerciseReleasesLinkPath = '/exercises/:exerciseId/:comesFromOwnExercisesSite/releases'
export const exerciseReleasesToLinkPath = '/exercises'

export function getExerciseReleaseLinkPath(exerciseId: number, comesFromOwnExercisesSite: boolean): string {
  return `${exerciseReleasesToLinkPath}/${exerciseId}/${comesFromOwnExercisesSite}/releases`
}

export const submissionsSiteLinkPath = `${exerciseReleasesLinkPath}/:releaseId/submissions`

export function getSubmissionsSiteToLinkPath(exerciseId: number, comesFromOwnExercisesSite: boolean,
                                             exerciseReleaseId: number
): string {
  return `${exerciseReleasesToLinkPath}/${exerciseId}/${comesFromOwnExercisesSite}/releases/${exerciseReleaseId}/submissions`
}


export const tutorViewSiteLinkPartPath = '/editor/tutor'
export const tutorViewSiteLinkMatchPath = '/editor/tutor/:comesFromOwnExercisesSite/:releaseId/:userId/:pLangId'

export function getTutorViewForSubmission(comesFromOwnExercisesSite: boolean | null, releaseId: number, userId: number,
                                          pLangId: number
): string {
  return `${tutorViewSiteLinkPartPath}/${comesFromOwnExercisesSite}/${releaseId}/${userId}/${pLangId}`
}

/**
 * this is only for the router to know the path
 * but cannot be used to programmatically navigate, use getSingleGroupLinkPath()
 * @type {string}
 */
export const singleGroupLinkPath = '/groups/:id'

export function getSingleGroupLinkPath(groupId: number): string {
  return `${groupsLinkPath}/${groupId}`
}

export const groupRolesLinkPath = '/roles/groups'
export const systemRolesLinkPath = '/roles/system'

export const ownSettingsLinkPath = '/settings'

const manageUsersLinkPathPart = '/manage/users'
//which should be one of the following 2
export const manageNewUsersLinkPath = `${manageUsersLinkPathPart}/new`
export const manageActivatedUsersLinkPath = `${manageUsersLinkPathPart}/activated`

export const manageTagsLinkPath = '/manage/tags'

export const aboutLinkPath = '/about'


export const exerciseEditorCreateNewLogoutHint = '/editor/exercises/create'
export const exerciseEditorCreateNewLinkPath = `${exerciseEditorCreateNewLogoutHint}/:comesFromOwnExercisesSite`

export function getExerciseEditorCreateNewLinkPath(comesFromOwnExercisesSite: boolean): string {
  return `/editor/exercises/create/${comesFromOwnExercisesSite}`
}


export const exerciseEditorChangeLogoutHelp = '/editor/exercises/edit'
export const exerciseEditorChangeLinkPath = `${exerciseEditorChangeLogoutHelp}/:comesFromOwnExercisesSite/:exerciseId`

export function getExerciseEditorEditToLink(exerciseId: number, comesFromOwnExercisesSite: boolean): string {
  return `/editor/exercises/edit/${comesFromOwnExercisesSite}/${exerciseId}`
}

export const doExerciseSiteLinkPath = '/doeditor/exercises/:releaseCode/:mode?'
export const doExerciseSiteToLinkPath = '/doeditor/exercises'

export function getDoExerciseSiteToLink(releaseCode: string): string {
  return `${doExerciseSiteToLinkPath}/${releaseCode}`
}


export const manageCustomProjectsSiteLinkMatchPath = '/manage/projects/own'


export const createCustomProjectSiteToLinkPath = '/manage/projects/own/create'

export const getCustomProjectLinkPath = '/manage/projects/own/edit'
export const getCustomProjectMatchLinkPath = '/manage/projects/own/edit/:customProjectId'

export function getCustomProjectToLinkPath(customProjectId: number): string {
  return `${getCustomProjectLinkPath}/${customProjectId}`
}

//if we have no targetUserId then use current user as targetUserId
export const assessmentStatisticsSiteLinkMatchPath = '/assessment/statistics/:targetUserId?'
export const assessmentStatisticsSiteToLinkPath = '/assessment/statistics'

export function getAssessmentStatisticsSiteToLinkPath(targetUserId: number): string {
  return `${assessmentStatisticsSiteToLinkPath}/${targetUserId}`
}

// ---- other links

export const submissionDownloadLinkPart = '/api/submissions/download/submission'

//TODO use id not code
/**
 * returns the download link for a specific submission
 * @param {string} releaseCode the release code
 * @param {number} userId the user id
 * @param {number} pLangId the p lang id
 * @returns {string} the link url
 */
export function getSubmissionDownloadLink(releaseCode: string, userId: number, pLangId: number): string {
  return `${debugOriginUrl}${submissionDownloadLinkPart}/${releaseCode}/${userId}/${pLangId}`
}

export function getAllSubmissionDownloadLink(releaseCode: string): string {
  return `${debugOriginUrl}${submissionDownloadLinkPart}/all/${releaseCode}`
}


export const assessmentResultsDownloadLinkPart = '/api/submissions/export/csv'

export function getAssessmentResultsDownloadLink(releaseCode: string, sortKey: keyof AssessmentFullBase,
                                                 ascending: boolean
): string {
  return `${debugOriginUrl}${assessmentResultsDownloadLinkPart}/${releaseCode}/${sortKey}/${ascending}`
}

export const testingLayerCallerName = 'testingLayer'

export const leftIconPopupOffset = 12

/**
 *
 * this is used in the markdown content (e.g. for an exercise description)
 * when we include pictures via [img not found](url) we need to specify an url
 * when we use a normal url and we move servers (e.g. from yapex.informatik... to localhost...)
 * the url would be invalid so we use an abstraction layer: asset:// is then translated to what we need
 * e.g. the file could be an inlined blob or we replace it with a real server url
 * because every context (exercise description / custom project description) knows which assets are connected to them
 * it is only necessary to specify the asset id and we get the exercise/custom project and can identify the asset uniquely
 * (not global but with the given context)
 * if we really want a global url the frontend can transform the url to an absolute url with the given context
 *
 * all without changing the markdown content
 *
 * SO wWE use MarkdownAssetFileUrlPrefix + asset id in the markdown e.g. asset://1
 * this is just here as an reminder, the user in the frontend should use this string when he wants to include an img
 * for the backend it has no real meaning (all the transforming is done on the frontend)
 */
export const assetPrefix = 'asset://'


export const doEditorEditorPanelViewEditorId = 'doEditorEditorPanelViewEditorId'
export const doEditorAfterEditorPanelViewEditorId = 'doEditorAfterEditorPanelViewEditorId'
export const customProjectEditorPanelViewEditorId = 'customProjectEditorPanelViewEditorId'
export const editExerciseChangeCodeTemplateViewEditorId = 'editExerciseChangeCodeTemplateViewEditorId'
export const editExerciseExerciseDescriptionEditorPanelViewEditorId = 'editExerciseExerciseDescriptionEditorPanelViewEditorId'
export const tutorViewEditorPanelViewEditorId = 'tutorViewEditorPanelViewEditorId'
export const tutorViewUserSolutionEditorPanelViewEditorId = 'tutorViewUserSolutionEditorPanelViewEditorId'
export const tutorViewUserAfterSolutionEditorPanelViewEditorId = 'tutorViewUserAfterSolutionEditorPanelViewEditorId'
export const tutorViewCodeTemplateEditorPanelViewEditorId = 'tutorViewCodeTemplateEditorPanelViewEditorId'
export const tutorViewTutorFeedbackForStudentEditor = 'tutorViewTutorFeedbackEditor'


/**
 * test type limits for
 * @see TestSettingsView
 */
export enum TestTypeLimits {
  IsNormalTest,
  IsSubmitTest,
  IsCustomTest,
  IsCustomProjectTest,
  IsJustRunProgram,
}

/**
 * we always get back the test server settings even if the backend cannot get the real settings...
 * then we get back this value to indicate this (without throwing an error)
 */
export const testServerPublicSettingsNotAvailableValue = -1

export const dashboardRefreshIntervalInS = 5

