/**
 * Created by janis dähne.
 */
import {FileWithData} from "../types/exerciseEditor";
import Logger from "./logger";
import {assetPrefix} from '../constants'


interface AssetWithUrl {
  readonly url: string,
  readonly asset: FileWithData
}

interface ExerciseAssetsStorage {
  // noinspection TsLint
  [assetId: string]: AssetWithUrl
}


// noinspection TsLint
let currentAssets: ExerciseAssetsStorage = {}


/**
 * tries to return the asset url from an asset
 * @param {FileWithData} asset
 * @returns {string}
 */
export function getObjectUrlForAsset(asset: FileWithData): string | null {

  // noinspection TsLint
  for (const key in currentAssets) {
    const tuple = currentAssets[key]

    if (tuple.asset.id === asset.id) {
      return tuple.url
    }
  }
  return null
}


/**
 * frees all asset resources
 *
 * use this also if the site is left (navigated away)
 *
 * TODO must this does not work... maybe the created dom img tag still holds a referenced??
 * --> working for exercise editor
 * or it's a browser bug? when watching the sources panel in chrome, the (no domain) tree we see the blob but calling
 * revokeObjectURL won't remove it
 */
export function freeAssetUrlsAndObjects(): void {

  // noinspection TsLint
  for (const key in currentAssets) {
    const tuple = currentAssets[key]
    URL.revokeObjectURL(tuple.url)

    Logger.debug("assetHelper", `revoking object url for asset (tuple) with id ${tuple.asset.id}`, tuple)
  }

  currentAssets = {}

}

/**
 * deletes all old object urls and releases resources, then creates new object urls for all given assets
 * only call this is it's really needed because we might have big assets for which we need to create new blobs here...
 * @param {ReadonlyArray<FileWithData>} assets
 */
export function createAndRefreshAssetUrlsAndObjects(assets: ReadonlyArray<FileWithData>): void {

  //revoke all old urls
  freeAssetUrlsAndObjects()

  // noinspection TsLint
  for (let i = 0; i < assets.length; i++) {
    const asset = assets[i]

    const bytes = new Uint8Array(asset.content);
    const blob = new Blob([bytes], {type: asset.mimeType})
    const url = URL.createObjectURL(blob)

    const tuple = {
      asset: asset,
      url: url
    }
    currentAssets[asset.id.toString()] = tuple

    Logger.debug("assetHelper", `creating object url for asset (tuple) with id ${tuple.asset.id}`, tuple)

  }
}

export function contentWithAssetsToContentWithUrlsFilter(contentWithAssetUrls: string): string {

  // noinspection TsLint
  let newContent = contentWithAssetUrls

  // noinspection TsLint
  for (const key in currentAssets) {
    const tuple = currentAssets[key]
    // newContent = newContent.split(tuple.asset.assetUrl).join(tuple.url);

    newContent = replaceAll(newContent, `${assetPrefix + tuple.asset.id}`, tuple.url)
  }

  return newContent
}

/**
 * returns the md markup for an image
 * TODO return other based on mime type...
 * @param asset
 */
export function getCopyAssetUrlText(asset: FileWithData): string {
  return `![not found](${assetPrefix + asset.id})`
}

//from https://stackoverflow.com/questions/1144783/how-to-replace-all-occurrences-of-a-string-in-javascript
function replaceAll(value: string, search: string, replacement: string): string {
  return value.split(search).join(replacement);
};
