import {AxiosResponse} from 'axios'
import {ResponseCodes} from './responseCodes'
import {ServerResponse} from '../types/response'
import {getI18n, KnownLangs, LangObj} from '../../i18n/i18nRoot'
import {errorDialog} from './dialogHelper'
import Logger from './logger'


export class ErrorHelper {


  /**
   * an error from the server with ok code but
   * some was wrong e.g. some field was null or emtpy...
   * @param e
   * @returns the message or null
   */
  static getErrorServerMessage(e: AxiosResponse): string | null {

    if (e &&  e.data && e.status === 200) {
        const data = e.data as ServerResponse

      if (data.code !== ResponseCodes.ok) {

        const msg = data.message

        if (msg && msg.trim() !== '') return msg
      }
    }

    return null
  }


  /**
   * use this only for server response errors ... else use a normal errorDialog
   * @param error
   * @param langId
   * @param title
   * @param message use null to only
   */
  static makeServerErrorDialog(error: AxiosResponse, langId: KnownLangs, title : keyof LangObj, message: keyof LangObj | null): void {

    if ((error as any as Error).message) {
      //this is a normal error no network/server error...
      Logger.error(`Error - ${(error as any as Error).message}, in catch: ${message}`, error)
      return
    }

    const serverErrorMsg = ErrorHelper.getErrorServerMessage(error)

    if (message === null) {
      errorDialog(getI18n(langId, title), `${getI18n(langId, 'Error')} - ${serverErrorMsg}`, langId)
      Logger.error(`Error - ${serverErrorMsg}`, error)
    }
    else {
      errorDialog(getI18n(langId, title), `${getI18n(langId, message)} - ${serverErrorMsg}`, langId)
      Logger.error(`${message} - ${serverErrorMsg}`, error)
    }

  }

}