/* tslint:disable */
/**
 * Created by janis dähne.
 */

import Logger from "./logger";
import throttle from 'lodash-es/throttle'
import {dateTimeReviverLocalTime} from "./jsonTransformer";
import {AssessmentFullBase} from "../types/submissions";
import {SortDirection} from "../types/sortDirection";
import {getInitialPaginationData, paginationPageSizeOptionAll} from '../constants'
import {AdditionalCsvColumn, CsvExportAssessmentOptionsMap} from '../types/csvExportAssessment'

//settings that are only stored at frontend

//1s should be ok, this is only that we save at MOST every 1s
//the save function is only triggered if something has changed
const throttleTimeInMs = 2000


interface SortingSettingTuple<T> {
  readonly sortByKey: keyof T
  readonly sortDirection: boolean | null
}

//TODO there is currently no handling when we change this...
const currentVersion = '2'

/**
 * use this as readonly and only change it through FrontendSettingsManager !!
 * normally we call the set funcs in the reducers...
 * for some special cases e.g. setCsvExportAssessmentsSettings (because it's not really possibly via a reducer)
 *
 * to apply the setting there are 2 good methods
 * 1. apply in componentWillMount (but keep in mind that e.g. dialogs could be temporarily displayed and the component is mounted again)
 * 2. apply in the reducer in the reset action
 */
type State = {

  /**
   * if we change the version string something might have changed... e.g. some props removed / added
   */
  version: string

  //--- various sorting

  groupExercisesGroupSorting: FrontendSortingElement[] | null

  //--- site tags filter displayed

  allOpenExercisesSiteTagsFilterIsDisplayed: boolean

  allClosedExercisesSiteTagsFilterIsDisplayed: boolean

  ownExercisesSiteTagsFilterIsDisplayed: boolean

  groupExercisesSiteTagsFilterIsDisplayed: boolean


  //--- store sortings

  submissionsSiteSorting: SortingSettingTuple<AssessmentFullBase> | null

  //---page size

  manageActivatedUsersPageSize: number
  manageNewUsersPageSize: number
  groupsPageSize: number
  singleGroupPageSize: number
  addMembersPageSize: number
  myProjectsPageSize: number
  ownExercisesPageSize: number
  groupExercisesPageSize: {[groupId: number]: number} //a map
  releasesViewPageSize: number
  submissionsViewPageSize: number
  assessmentStatisticPageSize: number
  openExercisesViaCodePageSize: number
  openExercisesViaVisibilityPageSize: number
  closedExercisesViaCodePageSize: number
  closedExercisesViaVisibilityPageSize: number

  /**
   * the do editor/exercise release id (only overwrite files from backend if frontend has newer ones (some crash)
   * AND it must be the same release id (else we would overwrite the wrong files...)
   *
   * the last known one
   */
  lastKnownDoEditorReleaseId: number

  /**
   * the current exercise releases that the user solves
   */
  doEditorReleaseId: number

  // /**
  //  * do editor files that should be saved (e.g. in case of a crash)
  //  *
  //  */
  // doEditorSolution: SolutionVersionDoExerciseFullBase | null

  /**
   * a frontend setting is only valid for one user
   * if the user changes destroy it
   */
  lastLoggedInUserId: number


  /**
   * true: use (js) diff to display wrong chars
   * false: not (only display actual output read
   */
  useDiffIfOutputMismatched: boolean

  /**
   * true: replace whitespaces in console input and output with bullets (·)
   * false: not replace whitespaces
   */
  useReplaceWhiteSpacesWithBullets: boolean

  /**
   * csv export assessments, only take the cols we find
   */
  csvExportAssessmentsSettings: CsvExportAssessmentOptionsMap
  /**
   * the csv export delimiter
   */
  csvExportDelimiter: string


  /**
   * the plang of the release we created last
   * or -x for invalid ... then pre select first
   */
  lastReleasedPLangId: number

  /**
   * selected group id when creating new exercise
   * -x for invalid ... then pre select last (because first is probably the default group)
   */
  lastNewExerciseGroupId: number

}

const initialPageSize = getInitialPaginationData().pageSize
const initial: State = {
  version: currentVersion,

  submissionsSiteSorting: null,
  groupExercisesGroupSorting: null,

  allOpenExercisesSiteTagsFilterIsDisplayed: false,
  allClosedExercisesSiteTagsFilterIsDisplayed: false,
  ownExercisesSiteTagsFilterIsDisplayed: false,
  groupExercisesSiteTagsFilterIsDisplayed: false,

  manageActivatedUsersPageSize: initialPageSize,
  manageNewUsersPageSize: initialPageSize,
  groupsPageSize: initialPageSize,
  singleGroupPageSize: initialPageSize,
  addMembersPageSize: initialPageSize,
  myProjectsPageSize: initialPageSize,
  ownExercisesPageSize: initialPageSize,
  groupExercisesPageSize: {},
  releasesViewPageSize: initialPageSize,
  submissionsViewPageSize: paginationPageSizeOptionAll, //server side paging is no real paging so loading all has almost no additional costs
  assessmentStatisticPageSize: paginationPageSizeOptionAll, //server side paging is no real paging so loading all has almost no additional costs
  openExercisesViaCodePageSize: initialPageSize,
  openExercisesViaVisibilityPageSize: initialPageSize,
  closedExercisesViaCodePageSize: initialPageSize,
  closedExercisesViaVisibilityPageSize: initialPageSize,

  lastLoggedInUserId: -1,
  lastKnownDoEditorReleaseId: -1,
  doEditorReleaseId: -1,
  // doEditorSolution: null,
  useDiffIfOutputMismatched: false,
  useReplaceWhiteSpacesWithBullets: true,

  csvExportAssessmentsSettings: {} as any,
  csvExportDelimiter: ',',

  lastReleasedPLangId: -1,
  lastNewExerciseGroupId: -1,
}

// noinspection TsLint
/**
 * represents the frontend settings that can be accessed globally
 *
 * the setters are normally called through the reducers
 * and in the reducers when we call reset we get the values from here (to overwrite reducer local initial values)
 *
 * because there is no explicit action call the frontend settings are transparent ... we can remove it and woul'd need
 * to change anything (except the calls to this class)
 *
 * TODO but maybe create a new reducer because we would load the settings from here in every reducer...
 * so we would get a lot of duplicate code...
 *
 * @type {State}
 */
let frontendSettings = initial

/**
 * not that all setting here might get cleared at any time ... e.g. when a user logs out (maybe another user logs in)
 */
export class FrontendSettingsManager {

  static readonly localStorageKey = "frontendSettings"
  static hasReadFrontendSettingsFromLocalStorage = false

  static maxByteLocalStorage = 4194304 //should be 4 mb?

  /**
   * sometimes some components set properties earlier than expected (e.g. multi tab editor set solution after
   * creating the session and thus changing the solution before we can check for a new frontend version)
   *
   * so lock the settings until we are ready to take changes
   * @type {boolean}
   */
  static isFrozen = false


  static setIsFrozen(isFrozen: boolean): void {
    Logger.debug('frontendSettingsManager', `setting isFrozen to ${isFrozen}`)
    FrontendSettingsManager.isFrozen = isFrozen
  }

  static getIsFrozen(): boolean {
    if (FrontendSettingsManager.isFrozen) {
      Logger.debug('frontendSettingsManager', `settings are frozen!`)
    }

    return FrontendSettingsManager.isFrozen
  }

  static set_groupExercisesGroupSorting(newSorting: FrontendSortingElement[] | null)  {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.groupExercisesGroupSorting = newSorting
    forceSave() //no throttling needed
  }

  //--- page size

  static set_manageActivatedUsersPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.manageActivatedUsersPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_manageNewUsersPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.manageNewUsersPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_groupsPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.groupsPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_singleGroupPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.singleGroupPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_addMembersPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.addMembersPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_myProjectsPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.myProjectsPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_ownExercisesPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.ownExercisesPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_groupExercisesPageSize(pageSize: number | null, groupId: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return

    if (pageSize === null) {
      delete frontendSettings.groupExercisesPageSize[groupId]
    } else {
      frontendSettings.groupExercisesPageSize[groupId] = pageSize
    }

    saveFrontendSettingsThrottled()
  }

  static set_releasesViewPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.releasesViewPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_submissionsViewPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.submissionsViewPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_assessmentStatisticPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.assessmentStatisticPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_openExercisesViaCodePageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.openExercisesViaCodePageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_openExercisesViaVisibilityPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.openExercisesViaVisibilityPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_closedExercisesViaCodePageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.closedExercisesViaCodePageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static set_closedExercisesViaVisibilityPageSize(pageSize: number): void {
    if (FrontendSettingsManager.getIsFrozen()) return
    frontendSettings.closedExercisesViaVisibilityPageSize = pageSize
    saveFrontendSettingsThrottled()
  }

  static setSubmissionsSiteSorting(sorting: SortingSettingTuple<AssessmentFullBase>): void {

    if (FrontendSettingsManager.getIsFrozen()) return

    frontendSettings.submissionsSiteSorting = sorting
    saveFrontendSettingsThrottled()
  }

  //--- is tags filter displayed

  static setAllOpenExercisesSiteTagsFilterIsDisplayed(isDisplayed: boolean): void {

    if (FrontendSettingsManager.getIsFrozen()) return

    frontendSettings.allOpenExercisesSiteTagsFilterIsDisplayed = isDisplayed
    saveFrontendSettingsThrottled()
  }

  static setAllClosedExercisesSiteTagsFilterIsDisplayed(isDisplayed: boolean): void {

    if (FrontendSettingsManager.getIsFrozen()) return

    frontendSettings.allClosedExercisesSiteTagsFilterIsDisplayed = isDisplayed
    saveFrontendSettingsThrottled()
  }

  static setOwnExercisesSiteTagsFilterIsDisplayed(isDisplayed: boolean): void {

    if (FrontendSettingsManager.getIsFrozen()) return

    frontendSettings.ownExercisesSiteTagsFilterIsDisplayed = isDisplayed
    saveFrontendSettingsThrottled()
  }

  static setGroupExercisesSiteTagsFilterIsDisplayed(isDisplayed: boolean): void {

    if (FrontendSettingsManager.getIsFrozen()) return

    frontendSettings.groupExercisesSiteTagsFilterIsDisplayed = isDisplayed
    saveFrontendSettingsThrottled()
  }


  //--- other
  static setUseDiffIfOutputMismatched(useDiffIfOutputMismatched: boolean): void {

    if (FrontendSettingsManager.getIsFrozen()) return

    frontendSettings.useDiffIfOutputMismatched = useDiffIfOutputMismatched
    saveFrontendSettingsThrottled()
  }

  static setUseReplaceWhiteSpacesWithBullets(useReplaceWhiteSpacesWithBullets: boolean): void {

    if (FrontendSettingsManager.getIsFrozen()) return

    frontendSettings.useReplaceWhiteSpacesWithBullets = useReplaceWhiteSpacesWithBullets
    saveFrontendSettingsThrottled()
  }

  // /**
  //  * ONLY PASS DOWN A COPY! else we could change the original reference
  //  * @param {SolutionVersionDoExerciseFullBase} doEditorSolution
  //  */
  // static setDoEditorSolution(doEditorSolution: SolutionVersionDoExerciseFullBase | null): void {
  //
  //   if (FrontendSettingsManager.getIsFrozen()) return
  //
  //   frontendSettings.doEditorSolution = doEditorSolution
  //   Logger.log("frontendSettingsManager", "saving frontend solution ... (but throttled")
  //   saveFrontendSettingsThrottled()
  // }

  static setLastKnownDoEditorReleaseId(lastKnownDoEditorReleaseId: number): void {

    if (FrontendSettingsManager.getIsFrozen()) return

    frontendSettings.lastKnownDoEditorReleaseId = lastKnownDoEditorReleaseId
    saveFrontendSettingsThrottled()
  }

  static setDoEditorReleaseId(doEditorReleaseId: number): void {

    if (FrontendSettingsManager.getIsFrozen()) return

    frontendSettings.doEditorReleaseId = doEditorReleaseId
    saveFrontendSettingsThrottled()
  }

  static setLastLoggedInUserId(lastLoggedInUserId: number): void {

    //if (FrontendSettingsManager.getIsFrozen()) return //always allow this

    frontendSettings.lastLoggedInUserId = lastLoggedInUserId
    saveFrontendSettingsThrottled()
  }

  static setCsvExportAssessmentsSettings( settings: CsvExportAssessmentOptionsMap, csvExportDelimiter: string): void {

    //if (FrontendSettingsManager.getIsFrozen()) return //always allow this

    frontendSettings.csvExportAssessmentsSettings = settings
    frontendSettings.csvExportDelimiter = csvExportDelimiter
    saveFrontendSettingsThrottled()
  }

  static setLastReleasedPLangId(lastReleasedPLangId: number): void {

    //if (FrontendSettingsManager.getIsFrozen()) return //always allow this

    frontendSettings.lastReleasedPLangId = lastReleasedPLangId
    saveFrontendSettingsThrottled()
  }

  static setLastNewExerciseGroupId(lastNewExerciseGroupId: number): void {

    //if (FrontendSettingsManager.getIsFrozen()) return //always allow this

    frontendSettings.lastNewExerciseGroupId = lastNewExerciseGroupId
    saveFrontendSettingsThrottled()
  }


  /**
   * when a new user logs in (user changed) or the exercise changed and thus the editor content is wrong/old
   * clear the storage
   */
  static clear(): void {
    frontendSettings = {...initial}
    forceDestroy();
  }

  static getFrontendSettings(): State {
    if (FrontendSettingsManager.hasReadFrontendSettingsFromLocalStorage === false) {
      FrontendSettingsManager.hasReadFrontendSettingsFromLocalStorage = true
      if (supportsLocalStorage()) {
        Logger.debug("frontendSettingsManager", "reading frontend settings")
        //this can fail... some inode error or something... then login and logout (works on linux)
        let json: string | null = null
        try {
          json = localStorage.getItem(FrontendSettingsManager.localStorageKey)
        } catch(err) {
            Logger.warn("couldn't access local storage (some inode error?)... maybe sing out and sign in in the os might help")
        }

        if (json) {
          try {
            frontendSettings = FrontendSettingsManager.setFrontendSettingsFromJsonString(json)
          } catch (err) {
            Logger.debug("frontendSettingsManager", "invalid settings")
          }

          if (frontendSettings.version !== currentVersion) {
              //version has changed... TODO??
          }

        } else {
          Logger.debug("frontendSettingsManager", "no settings found")
        }
      }
    }

    return frontendSettings
  }

  static forceSaveFrontendSettings(): void {
    forceSave();
  }

  static getFrontendSettingsAsJsonString(): string {

    return JSON.stringify(frontendSettings, (key: string, value) => {
      return value
    })
  }

  static setFrontendSettingsFromJsonString(json: string): State {
    //use same as in setup.ts to convert entities from backend (dateTimeReviverLocalTime converts dates to moment objs)
    const maybeSettings = JSON.parse(json, dateTimeReviverLocalTime)

    // if (maybeSettings && objectsHaveSameKeys(frontendSettings, maybeSettings)) {
    //   //TODO more checks ... also value types & ranges??
    //   //or just take valid keys (with values)
    //   frontendSettings = maybeSettings
    //   Logger.debug("frontendSettingsManager", "read frontend settings were valid (at least same keys)")
    // } else {
    //   Logger.warn( "[frontendSettingsManager] found invalid/old settings [not the same object keys]")
    // }

    //we now only take values that we know (e.g. if we create a property)
    if (maybeSettings) {
      for (const key in frontendSettings) {

        if (maybeSettings.hasOwnProperty(key)) {
          //take old value
          frontendSettings[key] = maybeSettings[key]
        }
        else {
          Logger.warn(` [frontendSettingsManager] settings key not found in old settings: ${key}, skipping`)
        }
      }
    }


    return frontendSettings
  }
}

/**
 * func to save the frontend settings but throttled
 * @type {() => void}
 */
const saveFrontendSettingsThrottled: () => void = _saveFrontendSettingsThrottled()


//from https://stackoverflow.com/questions/14368596/how-can-i-check-that-two-objects-have-the-same-set-of-property-names
// noinspection TsLint
function objectsHaveSameKeys(...objects: any[]): boolean {
  const allKeys = objects.reduce((keys, object) => keys.concat(Object.keys(object)), []);
  const union = new Set(allKeys);
  return objects.every(object => union.size === Object.keys(object).length);
}

function _saveFrontendSettingsThrottled(): () => void {
  // noinspection TsLint
  return throttle(() => {
    forceSave()
  }, throttleTimeInMs)
}

function forceDestroy(): void {
  if (supportsLocalStorage()) {
    localStorage.removeItem(FrontendSettingsManager.localStorageKey)
  }
}

function forceSave(): void {
  if (supportsLocalStorage()) {
    Logger.debug("frontendSettingsManager", "writing frontend settings")

    const json = FrontendSettingsManager.getFrontendSettingsAsJsonString()
    if (byteLength(json) >= FrontendSettingsManager.maxByteLocalStorage) {
      Logger.warn(
        'some browsers only support 5 MByte local storage ... the current settings are larger than 4 MByte! So the save might not work...')
    } else {
      //ok
    }
    localStorage.setItem(FrontendSettingsManager.localStorageKey, json)
  }
}

//from https://www.w3schools.com/html/html5_webstorage.asp
function supportsLocalStorage(): boolean {
  if (typeof(Storage) !== "undefined") {
    // Code for localStorage/sessionStorage.
    return true
  } else {
    // Sorry! No Web Storage support..
    return false
  }
}

//from https://stackoverflow.com/questions/5515869/string-length-in-bytes-in-javascript
// noinspection TsLint
function byteLength(str: string): number {
  // returns the byte length of an utf8 string
  // noinspection TsLint
  let s = str.length;
  // noinspection TsLint
  for (let i = str.length - 1; i >= 0; i--) {
    // noinspection TsLint
    let code = str.charCodeAt(i);
    // noinspection TsLint
    if (code > 0x7f && code <= 0x7ff) s++;
    // noinspection TsLint
    else { // noinspection TsLint
      if (code > 0x7ff && code <= 0xffff) s += 2;
    }
    // noinspection TsLint
    if (code >= 0xDC00 && code <= 0xDFFF) i--; //trail surrogate
  }
  return s;
}


/**
 * structure to store the display index for other objects (identified by id)
 * use the functions {@link frontendSettings_moveDisplaySortItemUp}, {@link frontendSettings_moveDisplaySortItemDown}, {@link frontendSettings_moveDisplaySortItemAbsolute} to change the properties in
 * {@link FrontendSettingsManager}
 */
type FrontendSortingElement = number

//--- use these functions to

/**
 * moves the given id in the {@param sortArray} up if possible
 * @param sortArray
 * @param id
 * @returns null if element with id was not found or out or range, else: new array with the result
 */
export function frontendSettings_moveDisplaySortItemUp(sortArray: FrontendSortingElement[] | null, id: number) : FrontendSortingElement[] | null {

  if (!sortArray) return null

  const index = sortArray.findIndex(p => p === id)
  if (index === -1 || index === 0) return null

  const newIndex = index-1

  let resultArray = Array.from(sortArray)

  const temp = resultArray[index]
  resultArray[index] = resultArray[newIndex]
  resultArray[newIndex] = temp

  return resultArray
}

/**
 * moves the given id in the {@param sortArray} down if possible
 * @param sortArray
 * @param id
 * @returns null if element with id was not found or out or range, else: new array with the result
 */
export function frontendSettings_moveDisplaySortItemDown(sortArray: FrontendSortingElement[] | null, id: number) : FrontendSortingElement[] | null {

  if (!sortArray) return null

  const index = sortArray.findIndex(p => p === id)
  if (index === -1 || index === sortArray.length-1) return null

  const newIndex = index+1

  let resultArray = Array.from(sortArray)

  const temp = resultArray[index]
  resultArray[index] = resultArray[newIndex]
  resultArray[newIndex] = temp

  return resultArray
}

/**
 * moves the given id in the {@param sortArray} absolute up/down if possible
 * @param sortArray
 * @param id
 * @param moveToFront true: absolute top (move to front), false: absolute bot (move to back)
 * @returns null if element with id was not found or out or range, else: new array with the result
 */
export function frontendSettings_moveDisplaySortItemAbsolute(sortArray: FrontendSortingElement[] | null, id: number, moveToFront: boolean): FrontendSortingElement[] | null {

  if (!sortArray) return null

  const index = sortArray.findIndex(p => p === id)
  if (index === -1) return null

  let resultArray = Array.from(sortArray)

  //move to back
  const temp = resultArray[index]
  resultArray.splice(index, 1)

  if (moveToFront) {
    resultArray.unshift(temp)
  } else {
    resultArray.push(temp)
  }

  return resultArray
}

type SyncResult = {
  result: FrontendSortingElement[]
  /**
   * something was changed, need to be saved into settings
   */
  anyChanges: boolean
}
/**
 * adds the new {@param ids} to {@param sortArray} and removes unused ids from {@param sortArray}
 * @param sortArray
 * @param ids this is the ground truth from the backend
 */
export function frontendSettings_syncDisplaySortingItems(sortArray: FrontendSortingElement[] | null, ids: ReadonlyArray<number>): SyncResult {

  let changed = false
  let resultArray = !sortArray ? [] : Array.from(sortArray)

  //use a try in case we change the frontend settings and this gets another type...
  try {
    //add all missing ids to the back of our old sortObj
    for(let i = 0; i < ids.length;i++) {
      const id = ids[i]
      if (resultArray.findIndex(p => p === id) === -1) {
        //new id is not found in sortObj
        resultArray.push(ids[i])
        changed = true
      }
    }

    //then remove all old entries in sortArray that are not longer found in ids
    for(let i = 0; i < resultArray.length;i++) {
      const id = resultArray[i]
      if (ids.findIndex(p => p === id) === -1) {
        resultArray.splice(i,1)
        changed = true
        i--
      }
    }
  } catch(err) {
    resultArray = []
    changed = true
  }


  return {
    result: resultArray,
    anyChanges: changed
  }

}
