/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./assessmentStatisticsSiteActionTypes";
import {notExhaustive} from "../_notExhausiveHelper";
import {
  AssessmentStatisticsFrontendOnly,
  AssessmentStatisticsFullBase,
  SingleAssessmentStatistic
} from "../../../types/assessmentStatistics";
import {SortDirection} from "../../../types/sortDirection";
import {
  AllActions as GetAssessmentStatisticsActions,
  reducer as getAssessmentStatistics
} from './getAssessmentStatistics'
import {PaginationStateExtend} from '../../../types/pagination'
import {getInitialPaginationData, paginationPageSizeOptionAll} from '../../../constants'
import {FrontendSettingsManager} from '../../../helpers/frontendSettingsManager'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'

export type State = {
  readonly assessmentStatistics: AssessmentStatisticsFrontendOnly | null

  readonly isLoading: boolean

  /**
   * the target user id to get the assessments for
   */
  readonly targetUserId: number

  readonly displaySubmitTestLine: boolean
  readonly displayNormalTestLine: boolean
  readonly displayManualPointsLine: boolean

} & PaginationStateExtend<SingleAssessmentStatistic>


export const initial: State = {
  assessmentStatistics: null,

  isLoading: false,

  targetUserId: -1,
  displaySubmitTestLine: true,
  displayNormalTestLine: true,
  displayManualPointsLine: true,

  pagination: {
    ...getInitialPaginationData(),
    orderByProp: 'createdAt',
    pageSize: paginationPageSizeOptionAll
  }
}

export interface SET_targetUserIdAction extends ActionBase {
  readonly type: ActionType.SET_targetUserId
  readonly targetUserId: number
}


//---pagination
export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
}

//---pagination end

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof SingleAssessmentStatistic
  readonly sortDirection: boolean | null
}

export interface SET_displaySubmitTestLineAction extends ActionBase {
  readonly type: ActionType.SET_displaySubmitTestLine
  readonly displaySubmitTestLine: boolean
}

export interface SET_displayNormalTestLineAction extends ActionBase {
  readonly type: ActionType.SET_displayNormalTestLine
  readonly displayNormalTestLine: boolean
}

export interface SET_displayManualPointsLineAction extends ActionBase {
  readonly type: ActionType.SET_displayManualPointsLine
  readonly displayManualPointsLine: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_sortByKeyAction
  | SET_displaySubmitTestLineAction
  | SET_displayNormalTestLineAction
  | SET_displayManualPointsLineAction

  | GetAssessmentStatisticsActions
  | SetSearchTextAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction
  | SET_targetUserIdAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {


    case ActionType.GET_assessmentStatistics_PENDING:
    case ActionType.GET_assessmentStatistics_FULFILLED:
    case ActionType.GET_assessmentStatistics_REJECTED:
      return getAssessmentStatistics(state, action)

    case ActionType.SET_displaySubmitTestLine:
      return {
        ...state,
        displaySubmitTestLine: action.displaySubmitTestLine
      }

    case ActionType.SET_displayNormalTestLine:
      return {
        ...state,
        displayNormalTestLine: action.displayNormalTestLine
      }
    case ActionType.SET_displayManualPointsLine:
      return {
        ...state,
        displayManualPointsLine: action.displayManualPointsLine
      }

    case ActionType.SET_targetUserId:
      return {
        ...state,
        targetUserId: action.targetUserId
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          orderByProp: action.sortByKey,
          isOrderByDesc: action.sortDirection
        }
      }

    case ActionType.setSearchText:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          searchText: action.searchText
        }
      }
    case ActionType.setPaginationActivePage:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          currentPage: action.activePage
        }
      }

    case ActionType.setPaginationPageSize:

      FrontendSettingsManager.set_assessmentStatisticPageSize(action.pageSize)

      return {
        ...state,
        pagination: {
          ...state.pagination,
          pageSize: action.pageSize,
          currentPage: 1,
        }
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
        pagination: {
          ...initial.pagination,
          pageSize: FrontendSettingsManager.getFrontendSettings().assessmentStatisticPageSize
        }
      }

    default:
      //enable if we have > 2 actions
      notExhaustive(action)
      return state
  }
}

