import {SystemSettingsFromBackend, TestServerPublicSettings} from "../../../types/systemSettings";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from './globalSystemSettingsActionTypes'

import {
  AllActions as AllGetGlobalSystemSettingsActions,
  reducer as getGlobalSystemSettingsReducer
} from './getSystemSettingsReducer'

import {AllActions as GetServermessageActions, reducer as getServerMessageReducer} from './getServerMessageReducer'
import {AllActions as AllUpdateSystemSettingsActions} from '../systemSettingsSite/crud/updateSystemSettingsReducer'
import {ActionType as SystemSettingsSiteActionTypes} from '../systemSettingsSite/systemSettingsSiteActionTypes'
import {
  AllActions as AllGetTestServerPublicSettingsReducer,
  reducer as getTestServerPublicSettingsReducer
} from './getTestServerPublicSettingsReducer'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'
import {testServerPublicSettingsNotAvailableValue} from '../../../constants'


//use this for the global system settings, this is different from the system settings site (else we would live edit...)
//also saving/loading the system settings site should affect this loading
export type State = {
  readonly isLoading: boolean
  readonly systemSettings: SystemSettingsFromBackend

  /**
   * test server public settings we need needs some props from this
   * this does not need to be soooo current like other values because it's not likely to change
   *
   * if the test server is down we get back a response from backend with initial values -1 which should signal that there is some error/problem
   */
  readonly testServerPublicSettings: TestServerPublicSettings

  /**
   * used to know if the user already read the server message (then disable animations... and some visuals)
   */
  readonly clickedOnServerMessage: boolean
  /**
   * if the server message changes then we need to set
   * @see clickedOnServerMessage again to false (even if it was true)
   *
   * is set when we click on the message
   * @sse clickedOnServerMessage
   */
  readonly lastKnownServerMessage: string

  /**
   * a local message for the client (normally a message about the server e.g. the server is currently offline)
   * display this separate from the normal server messages
   */
  readonly localServerMessage: string

  readonly clickedOnLocalServerMessage: boolean

  /**
   * see
   * @lastKnownServerMessage but for
   * @localServerMessage
   */
  readonly lastKnownLocalServerMessage: string

  /**
   * the server message from the server
   * use this to not interfere getting
   * @systemSettings on some sites
   *
   * SO USE this instead of
   * @systemSettings.currentServerMessage
   */
  readonly serverMessage: string
}


export const initial: State = {
  isLoading: false,
  systemSettings: {
    maxNumberOfTestsWithOneRequest: 1,
    maxCustomProjectsPerUser: -1,
    maxCustomTestsPerParticipation: -1,
    currentServerMessage: '',
    maxNumberOfTestsWithOneRequestSubmitTestServer: -1,
    submitTestServerTimeoutInMs: -1,
    submitTestServerUrl: '',
    testServerStatsUrl: '',
    submitTestServerStatsUrl: '',
    testServerTimeoutInMs: -1,
    testServerUrl: '',
    testServerConfigUiUrl: '',
    submitTestServerConfigUiUrl: '',
    justRunProgramMaxDiskSpaceInKb: -1,
    justRunProgramMemoryLimitInKb: -1,
    justRunProgramTimeoutInMs: -1,
    justRunProgramCompileTimeoutInMs: -1,
    customProjectTestTimeoutInMs: -1,
    customProjectTestMemoryLimitInKb: -1,
    customProjectTestMaxDiskSpaceInKb: -1,
    customProjectTestCompileTimeoutInMs: -1,
    defaultGroupCreatorGroupRoleId: -1,
    defaultGroupRoleId: -1,
    defaultUserGroupId: -1,
    testServerTraceLogApiUrl: '',
  },
  testServerPublicSettings: {
    hardTimeoutInMs: testServerPublicSettingsNotAvailableValue,
    hardCompileTimeoutInMs: testServerPublicSettingsNotAvailableValue,
    hardMemoryLimitInKb: testServerPublicSettingsNotAvailableValue,
    hardDiskSpaceLimitInKb: testServerPublicSettingsNotAvailableValue,
    maxNumberOfTestsWithOneRequest: testServerPublicSettingsNotAvailableValue
  },
  clickedOnServerMessage: false,
  lastKnownServerMessage: '',
  localServerMessage: '',
  serverMessage: '',
  clickedOnLocalServerMessage: false,
  lastKnownLocalServerMessage: ''
}


export interface SET_clickedOnServerMessageAction extends ActionBase {
  readonly type: ActionType.SET_clickedOnServerMessage
  readonly clicked: boolean
}

export interface SET_clickedOnLocalServerMessageAction extends ActionBase {
  readonly type: ActionType.SET_clickedOnLocalServerMessage
  readonly clicked: boolean
}

export interface SET_localServerMessageAction extends ActionBase {
  readonly type: ActionType.SET_localServerMessage
  readonly message: string
  readonly forceResetClickedState: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | AllGetGlobalSystemSettingsActions
  | AllGetTestServerPublicSettingsReducer
  | AllUpdateSystemSettingsActions
  | GetServermessageActions

  | SET_clickedOnServerMessageAction
  | SET_localServerMessageAction
  | SET_clickedOnLocalServerMessageAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    //also listen when the system settings get the new system settings
    case SystemSettingsSiteActionTypes.UPDATE_systemSettings_PENDING:
      return state
    case SystemSettingsSiteActionTypes.UPDATE_systemSettings_FULFILLED:
      return {
        ...state,
        systemSettings: action.payload,
        serverMessage: action.payload.currentServerMessage,
        clickedOnServerMessage: state.lastKnownServerMessage === action.payload.currentServerMessage
                                ? state.clickedOnServerMessage //if we already know the message, then keep the state
                                : false //else reset
      }
    case SystemSettingsSiteActionTypes.UPDATE_systemSettings_REJECTED:
      return state


    case ActionType.GET_systemSettings_PENDING:
    case ActionType.GET_systemSettings_FULFILLED:
    case ActionType.GET_systemSettings_REJECTED:
      return getGlobalSystemSettingsReducer(state, action)

    case ActionType.GET_testServerPublicSettings_PENDING:
    case ActionType.GET_testServerPublicSettings_FULFILLED:
    case ActionType.GET_testServerPublicSettings_REJECTED:
      return getTestServerPublicSettingsReducer(state, action)

    case ActionType.GET_serverMessage_PENDING:
    case ActionType.GET_serverMessage_FULFILLED:
    case ActionType.GET_serverMessage_REJECTED:
      return getServerMessageReducer(state, action)


    case ActionType.SET_clickedOnServerMessage:
      return {
        ...state,
        clickedOnServerMessage: action.clicked,
        lastKnownServerMessage: state.systemSettings.currentServerMessage
      }

    case ActionType.SET_clickedOnLocalServerMessage:
      return {
        ...state,
        clickedOnLocalServerMessage: action.clicked,
        lastKnownLocalServerMessage: state.localServerMessage
      }

    case ActionType.SET_localServerMessage:

      //do nothing because this is set on every request...
      //because a successful request indicates we need to remove the localServerMessage
      if (state.localServerMessage === action.message && action.message === '') return state


      // noinspection TsLint
      let clickedOnLocalServerMessage = state.lastKnownLocalServerMessage === action.message
                                        ? state.clickedOnLocalServerMessage //if we already know the message, then keep the state
                                        : false //else reset

      if (action.forceResetClickedState) {
        clickedOnLocalServerMessage = false
      }

      return {
        ...state,
        localServerMessage: action.message,
        clickedOnLocalServerMessage: clickedOnLocalServerMessage
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

