/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {SelectableUserFrontendExclusive} from "../../../types/users";
import {ActionBase, ActionType} from "./manageActivatedUsersSiteActionTypes";
import {AllActions as GetActions, reducer as getReducer} from './getActivatedUsersReducer'
import {AllActions as GetSystemRolesActions, reducer as systemRolesReducer} from './getSystemRolesReducer'
import {AllActions as DeleteActions, reducer as deleteReducer} from './deleteActivatedUserReducer'
import {SystemRoleFullBase} from "../../../types/userData";
import {SortDirection} from "../../../types/sortDirection";
import {getInitialPaginationData, paginationPageSizeOptions} from '../../../constants'
import {PaginationStateExtend, PaginationStateData} from '../../../types/pagination'
import {PaginationHelper} from '../../../helpers/paginationHelper'
import {FrontendSettingsManager} from '../../../helpers/frontendSettingsManager'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'


export type State = {
  /**
   * the activated users
   */
  readonly activatedUsers: ReadonlyArray<SelectableUserFrontendExclusive>
  readonly systemRoles: ReadonlyArray<SystemRoleFullBase>
  readonly isLoading: boolean
  readonly isChangeUserDataDialogDisplayed: boolean

} & PaginationStateExtend<SelectableUserFrontendExclusive>

export const initial: State = {
  activatedUsers: [],
  systemRoles: [],
  isLoading: false,
  isChangeUserDataDialogDisplayed: false,

  pagination: {
    ...getInitialPaginationData(),
    orderByProp: 'lastName',

  }
}


export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
}

export interface SET_isChangeUserDataDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeUserDataDialogDisplayed
  readonly isChangeUserDataDialogDisplayed: boolean
}

export interface SET_activatedUser_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_activatedUser_uiIsSelected
  readonly newUserId: number
  readonly isSelected: boolean
}

export interface SET_all_activatedUsers_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_all_activatedUsers_uiIsSelected
  readonly isSelected: boolean
}


export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof SelectableUserFrontendExclusive
  readonly sortDirection: boolean | null
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_isChangeUserDataDialogDisplayedAction
  | GetActions
  | GetSystemRolesActions
  | SET_activatedUser_uiIsSelectedAction
  | SET_all_activatedUsers_uiIsSelectedAction
  | DeleteActions
  | SET_sortByKeyAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction
  | SetSearchTextAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_activatedUsers_PENDING:
    case ActionType.GET_activatedUsers_FULFILLED:
    case ActionType.GET_activatedUsers_REJECTED:
      return getReducer(state, action)


    case ActionType.GET_systemRoles_PENDING:
    case ActionType.GET_systemRoles_FULFILLED:
    case ActionType.GET_systemRoles_REJECTED:
      return systemRolesReducer(state, action)

    case ActionType.DELETE_activatedUser_PENDING:
    case ActionType.DELETE_activatedUser_FULFILLED:
    case ActionType.DELETE_activatedUser_REJECTED:
      return deleteReducer(state, action)

    case ActionType.SET_activatedUser_uiIsSelected:
      return {
        ...state,
        activatedUsers: state.activatedUsers.map(
          user => user.id === action.newUserId
                  ?
            {
              ...user,
              uiIsSelected: action.isSelected
            }
                  : user)
      }

    case ActionType.SET_all_activatedUsers_uiIsSelected:
      return {
        ...state,
        activatedUsers: state.activatedUsers.map(user => {
          return {
            ...user,
            uiIsSelected: action.isSelected
          }
        })
      }


    case ActionType.setSearchText:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          searchText: action.searchText
        }
      }
    case ActionType.setPaginationActivePage:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          currentPage: action.activePage
        }
      }

    case ActionType.setPaginationPageSize:

      FrontendSettingsManager.set_manageActivatedUsersPageSize(action.pageSize)

      return {
        ...state,
        pagination: {
          ...state.pagination,
          pageSize: action.pageSize,
          currentPage: 1,
        }
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          orderByProp: action.sortByKey,
          isOrderByDesc: action.sortDirection
        }
      }

    case ActionType.SET_isChangeUserDataDialogDisplayed:
      return {
        ...state,
        isChangeUserDataDialogDisplayed: action.isChangeUserDataDialogDisplayed
      }
    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
        pagination: {
          ...initial.pagination,
          pageSize: FrontendSettingsManager.getFrontendSettings().manageActivatedUsersPageSize
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

