import {PLangFullBase, PLangWithExtrasFromBackend} from "../../../types/pLangs";
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from './pLangsSettingsViewActionTypes'

import {AllActions as AllGetPLangActions, reducer as getPLangsReducer} from './crud/getPLangsReducer'
import {
  AllActions as AllGetPLangsWithExtraDataActions,
  reducer as getPLangsWithExtraDataReducer
} from './crud/getPLangsWithExtraDataReducer'
import {AllActions as AllCreatePLangsActions, reducer as createPLangReducer} from './crud/createPLangReducer'
import {AllActions as AllUpdatePLangsActions, reducer as updatePlangsReducer} from './crud/updatePlangsReducer'
import {AllActions as AllDeletePLangsActions, reducer as deletePLangReducer} from './crud/deletePLangReducer'
import Logger from '../../../helpers/logger'
import {getNextFrontendId} from '../../../helpers/frontendIdHelper'
import {ActionTypeGlobal, RESET_GlobalAction} from '../globalActions'


export type State = {
  readonly pLangs: ReadonlyArray<PLangFullBase>


  readonly isLoading: boolean

  /**
   * used by the modify plangs view
   */
  readonly pLangsWithExtraData: ReadonlyArray<PLangWithExtrasFromBackend>

  /**
   * the plang id we currently edit
   * null to reset
   * <= 0 to create new
   */
  readonly editingPLangId: number | null

  /**
   * a copy used for editing
   * we can then rollback to original array or commit to server
   */
  readonly pLangCopy: PLangWithExtrasFromBackend | null
}

export const initial: State = {
  pLangs: [],
  pLangsWithExtraData: [],
  isLoading: false,
  editingPLangId: null,
  pLangCopy: null
}


export interface SET_editingPLangIdAction extends ActionBase {
  readonly type: ActionType.SET_editingPLangId
  /**
   * use -x or 0 to indicate to create a new plang
   */
  readonly editingPLangId: number | null
}

//--- change plang props

export interface SET_pLang_displayNameAction extends ActionBase {
  readonly type: ActionType.SET_pLang_displayName
  readonly displayName: string
}

export interface SET_pLang_internalNameAction extends ActionBase {
  readonly type: ActionType.SET_pLang_internalName
  readonly internalName: string
}

export interface SET_pLang_fileExtensionsWithDotAction extends ActionBase {
  readonly type: ActionType.SET_pLang_fileExtensionsWithDot
  readonly fileExtensionsWithDot: string
}

export interface SET_pLang_editorHighlightModeNameAction extends ActionBase {
  readonly type: ActionType.SET_pLang_editorHighlightModeName
  readonly editorHighlightModeName: string
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | AllGetPLangActions
  | AllGetPLangsWithExtraDataActions

  | AllCreatePLangsActions
  | AllUpdatePLangsActions
  | AllDeletePLangsActions

  | SET_editingPLangIdAction

  | SET_pLang_displayNameAction
  | SET_pLang_internalNameAction
  | SET_pLang_fileExtensionsWithDotAction
  | SET_pLang_editorHighlightModeNameAction

  | RESET_GlobalAction

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {


    case ActionType.GET_pLangs_PENDING:
    case ActionType.GET_pLangs_FULFILLED:
    case ActionType.GET_pLangs_REJECTED:
      return getPLangsReducer(state, action)

    case ActionType.GET_pLangsWithExtraData_PENDING:
    case ActionType.GET_pLangsWithExtraData_FULFILLED:
    case ActionType.GET_pLangsWithExtraData_REJECTED:
      return getPLangsWithExtraDataReducer(state, action)

    case ActionType.CREATE_pLang_PENDING:
    case ActionType.CREATE_pLang_FULFILLED:
    case ActionType.CREATE_pLang_REJECTED:
      return createPLangReducer(state, action)

    case ActionType.UPDATE_pLangs_PENDING:
    case ActionType.UPDATE_pLangs_FULFILLED:
    case ActionType.UPDATE_pLangs_REJECTED:
      return updatePlangsReducer(state, action)

    case ActionType.DELETE_pLang_PENDING:
    case ActionType.DELETE_pLang_FULFILLED:
    case ActionType.DELETE_pLang_REJECTED:
      return deletePLangReducer(state, action)


    case ActionType.SET_editingPLangId:


      //reset
      if (action.editingPLangId === null) {
        return {
          ...state,
          editingPLangId: null,
          pLangCopy: null
        }
      }

      //action.editingPLangId is !== null here
      if (action.editingPLangId <= 0) {
        const newId = getNextFrontendId()
        return {
          ...state,
          editingPLangId: newId,
          pLangCopy: {
            id: newId,
            displayName: '',
            internalName: '',
            fileExtensionsWithDot: '',
            editorHighlightModeName: '',
            canBeDeleted: false,
            cannotBeDeletedReason: null,
          }
        }
      }


      //make sure we update the plangs first before ending the edit

      const targetPLang = state.pLangsWithExtraData.find(p => p.id === action.editingPLangId)

      if (!targetPLang) {
        Logger.warn(`could not find plang with id ${action.editingPLangId} to edit`)
        return state
      }

      return {
        ...state,
        editingPLangId: action.editingPLangId,
        pLangCopy: {
          ...targetPLang
        }
      }


    case ActionType.SET_pLang_displayName:
      return {
        ...state,
        pLangCopy: {
          ...state.pLangCopy,
          displayName: action.displayName
        }
      }
    case ActionType.SET_pLang_internalName:
      return {
        ...state,
        pLangCopy: {
          ...state.pLangCopy,
          internalName: action.internalName
        }
      }
    case ActionType.SET_pLang_fileExtensionsWithDot:
      return {
        ...state,
        pLangCopy: {
          ...state.pLangCopy,
          fileExtensionsWithDot: action.fileExtensionsWithDot
        }
      }
    case ActionType.SET_pLang_editorHighlightModeName:
      return {
        ...state,
        pLangCopy: {
          ...state.pLangCopy,
          editorHighlightModeName: action.editorHighlightModeName
        }
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial
      }

    default:
      notExhaustive(action)
      return state
  }
}

