/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../../_notExhausiveHelper";
import {ActionBase, ActionType} from "./addsUserDialogActionTypes";
import {AddableUserFrontendExclusive, UserPreviewFromBackend} from "../../../../../types/users";
import {AllActions as GetUsersActions, reducer as getUsersReducer} from './getUsersCrudReducer'
import {AllActions as AddMembersActions, reducer as addMembersReducer} from './addMembersReducer'
import {SelectableGroupUserFrontend} from "../../../../../types/group";
import {SortDirection} from "../../../../../types/sortDirection";
import {PaginationStateData, PaginationStateExtend} from '../../../../../types/pagination'
import {getInitialPaginationData} from '../../../../../constants'
import {FrontendSettingsManager} from '../../../../../helpers/frontendSettingsManager'
import {ActionTypeGlobal, RESET_GlobalAction} from '../../../globalActions'


export type State = {
  /**
   * all possible users that could be added to a group
   * the system roles are not set!
   */
  readonly users: ReadonlyArray<AddableUserFrontendExclusive>


  /**
   * true: if at least 1 user is selected,
   * false: else
   */
  readonly isValid: boolean

  readonly isBusy: boolean

  readonly sortByKey: keyof AddableUserFrontendExclusive
  readonly sortDirection: SortDirection

} & PaginationStateExtend<UserPreviewFromBackend>

export const initial: State = {
  users: [],
  isValid: false,
  isBusy: false,
  sortByKey: 'lastName',
  sortDirection: SortDirection.ascending,

  pagination: {
    ...getInitialPaginationData(),
    orderByProp: 'lastName'
  }
}

//---pagination
export interface SetSearchTextAction extends ActionBase {
  readonly type: ActionType.setSearchText
  readonly searchText: string
}

export interface SetActivePaginationPageAction extends ActionBase {
  readonly type: ActionType.setPaginationActivePage
  readonly activePage: number
}

export interface SetPaginationPageSizeAction extends ActionBase {
  readonly type: ActionType.setPaginationPageSize
  readonly pageSize: number
}

//---pagination end

export interface SET_user_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_user_uiIsSelected
  readonly isSelected: boolean
  readonly user: AddableUserFrontendExclusive
}

export interface SET_all_users_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_all_users_uiIsSelected
  readonly isSelected: boolean
}

export interface INVERT_allSelectionsAction extends ActionBase {
  readonly type: ActionType.INVERT_allSelections
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof UserPreviewFromBackend
  readonly sortDirection: boolean | null
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_user_uiIsSelectedAction
  | SET_all_users_uiIsSelectedAction
  | GetUsersActions
  | INVERT_allSelectionsAction
  | AddMembersActions
  | SET_sortByKeyAction
  | SetSearchTextAction
  | SetActivePaginationPageAction
  | SetPaginationPageSizeAction

  | RESET_GlobalAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_users_PENDING:
    case ActionType.GET_users_FULFILLED:
    case ActionType.GET_users_REJECTED:
      return getUsersReducer(state, action)

    case ActionType.ADD_members_PENDING:
    case ActionType.ADD_members_FULFILLED:
    case ActionType.ADD_members_REJECTED:
      return addMembersReducer(state, action)

    case ActionType.SET_user_uiIsSelected:
      return {
        ...state,
        users: state.users.map(user => {
          return user.id === action.user.id
                 ?
            {
              ...user,
              uiIsSelected: action.isSelected
            }
                 : user
        })
      }

    case ActionType.SET_all_users_uiIsSelected:
      return {
        ...state,
        users: state.users.map(user => {
          return {
            ...user,
            uiIsSelected: user.isAlreadyMember
                          ? false
                          : action.isSelected
          }
        })
      }

    case ActionType.INVERT_allSelections:
      return {
        ...state,
        users: state.users.map(user => {
          return {
            ...user,
            uiIsSelected: user.isAlreadyMember
                          ? false
                          : !user.uiIsSelected
          }
        })
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          orderByProp: action.sortByKey,
          isOrderByDesc: action.sortDirection
        }
      }

    case ActionType.setSearchText:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          searchText: action.searchText
        }
      }
    case ActionType.setPaginationActivePage:
      return {
        ...state,
        pagination: {
          ...state.pagination,
          currentPage: action.activePage
        }
      }

    case ActionType.setPaginationPageSize:

      FrontendSettingsManager.set_addMembersPageSize(action.pageSize)

      return {
        ...state,
        pagination: {
          ...state.pagination,
          pageSize: action.pageSize,
          currentPage: 1,
        }
      }

    case ActionType.RESET:
    case ActionTypeGlobal.RESET_Global:
      return {
        ...initial,
        pagination: {
          ...initial.pagination,
          pageSize: FrontendSettingsManager.getFrontendSettings().addMembersPageSize,
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

