/**
 * Created by janis dähne.
 */


import {ReleaseDurationType} from "./ReleaseDurationType";
import {ReleaseStartType} from "./ReleaseStartType";
import {Moment} from "moment";
import {GroupPreviewFromBackend, UserGroupBase, UserGroupFromBackend} from "./group";
import {GroupRoleWithPermissionFullBase} from "./groupRoles";
import {TagFullBase} from "./tag";
import {PaginatedData, PaginationStateData} from './pagination'

/**
 * the state of an exercise (preview)
 */
export enum ExerciseState {
  /**
   * the exercise is open to be worked on
   */
  open,
  /**
   * the exercise is closed (not longer released or working time is up)
   */
  closed
}

/**
 * a preview for an exercise (just some basic information) for the users (students) to choose an exercise to solve
 */
export interface ExercisePreviewFromBackend {
  readonly id: number
  /**
   * the display name
   */
  readonly displayName: string
  /**
   * the short description
   */
  readonly shortDescription: string

  /**
   * the code from the related exercise release
   */
  readonly generatedCode: string

  /**
   * true: assessment result hould not count, false: not
   */
  readonly shouldNotCount: boolean

  /**
   * exercise and all dependent entities should be readonly (no edit/no delete)!
   */
  readonly isPermanentlyLocked: boolean
  /**
   * the plang the release is for
   */
  readonly releasedForPLangId: number

  /**
   * the tags connected to this exercise
   */
  readonly tags: ReadonlyArray<TagFullBase>


  /**
   * the release start type (we cannot rely on releaseDurationType to check if the release will end)
   * if the release start type is automatic then the releaseDurationType should be taken into account
   */
  readonly releaseStartType: ReleaseStartType

  /**
   *  the duration type for the release
   */
  readonly releaseDurationType: ReleaseDurationType //see Constants.ReleaseDurationTypes

  /**
   * the automatic ending time (if the releaseDurationType is automatic)
   */
  readonly automaticEndAt: Moment | null

  /**
   * if true apply the availableWorkingTimeInMinutes else unlimited working time
   */
  readonly hasLimitedWorkingTime: boolean

  /**
   * the time the student can spend at the exercise
   */
  readonly availableWorkingTimeInMinutes: number

  /**
   * true: the exercise is visible to all users event if they not entered the code, false: not
   */
  readonly isVisibleToAllAfterRelease: boolean

  /**
   * the date time when the user has accessed the exercise for the first time
   * (not actually accessed but could have accessed e.g. wehn he/she enters the release code)
   * used to calculate the left working time...
   * can be null because when accessed without code we know nothing about the (future) participation
   */
  readonly firstAccessedAt: Moment | null

  /**
   * true: is still released,false: not
   */
  readonly isReleased: boolean
}

/**
 * a exercise preview for the sites that displays the available exercises in a group or where the user is the creator,
 * no release data needed
 */
export interface EditableExercisePreviewFromBackend {
  readonly id: number
  /**
   * the display name
   */
  readonly displayName: string
  /**
   * the short description
   */
  readonly shortDescription: string

  /**
   * the tags connected to this exercise
   */
  readonly tags: ReadonlyArray<TagFullBase>


  /**
   * true: only visible to me in the groups exercise overview, false: do not show to others in group ...
   */
  readonly isOnlyVisibleToMe: boolean

  /**
   * exercise and all dependent entities should be readonly (no edit/no delete)!
   */
  readonly isPermanentlyLocked: boolean

  /**
   * when the exercise was last updated
   */
  readonly lastUpdatedAt: Moment

  /**
   * the amount of solutions submitted for this exercise
   */
  readonly solutions: number

  /**
   * the amount of releases for this exercise (used to determine of
   * the exercise can be deleted (when releases == 0) or not)
   */
  readonly releases: number

  /**
   * the managing group id to get the users permission for that group
   */
  readonly managingGroupId: number

  /**
   * the plang ids the exercise supports (has code templates for)
   */
  readonly supportedPLangIds: ReadonlyArray<number>

  /**
   * the owner id (if this is null the exercise has really no user)
   */
  readonly ownerId: number | null

  /**
   * the owner of the exercise or null
   * (can be null if not loaded... currently we load always but we could use lazy loading e.g. only on click or something because getting user in db query for exercise is expensive)
   */
  readonly owner: ExerciseOwner | null
}

export interface ExerciseOwner {
  readonly id: number
  readonly token: string
  readonly firstName: string
  readonly lastName: string
}


export interface GroupPreviewExerciseTupleFrontendOnly {

  readonly userGroup: UserGroupFromBackend

  /**
   * the state obj for this group exercises preview
   */
  readonly pagination: PaginationStateData<EditableExercisePreviewFromBackend>
  /**
   * the accessible exercise previews in the group
   */
  readonly exercisePreviews: ReadonlyArray<EditableExercisePreviewFromBackend>

  /**
   * a spinner only for this group
   */
  readonly isLoading: boolean
}


export interface SingleGroupPreviewExerciseTupleFromBackend {
  /**
   * the group (preview)
   */
  readonly groupPreview: GroupPreviewFromBackend

  /**
   * we need the group role for the current user in the group (groupPreview) to determine what he/she is allowed to do
   */
  readonly groupRole: GroupRoleWithPermissionFullBase
  /**
   * the exercise previews in the group
   */
  readonly exercisePreview: EditableExercisePreviewFromBackend
}
